@@ -99,8 +99,9 @@ QString KalziumUtils::prettyUnit( const Element* el, ChemicalDataObject::BlueObe
                     result = i18n( "Unknown Value" );
                 else
                 {
-                    val = TempUnit::convert( val, (int)TempUnit::Kelvin, Prefs::temperature() );
-                    result = i18nc( "%1 is the temperature, %2 is the unit, like \"300 K\"", "%1 %2", val, TempUnit::unitListSymbol( Prefs::temperature() ) );
+                    val = TempUnit::convert( val, (int)TempUnit::Kelvin, Prefs::temperatureUnit() );
+                    result = i18nc( "%1 is the temperature, %2 is the unit, like \"300 K\"", 
+                            "%1 %2", val, TempUnit::unitListSymbol( Prefs::temperatureUnit() ) );
                 }
                 break;
             }
@@ -111,8 +112,9 @@ QString KalziumUtils::prettyUnit( const Element* el, ChemicalDataObject::BlueObe
                     result = i18n( "Unknown Value" );
                 else
                 {
-                    val = TempUnit::convert( val, (int)TempUnit::Kelvin, Prefs::temperature() );
-                    result = i18nc( "%1 is the temperature, %2 is the unit, like \"300 K\"", "%1 %2", val, TempUnit::unitListSymbol( Prefs::temperature() ) );
+                    val = TempUnit::convert( val, (int)TempUnit::Kelvin, Prefs::temperatureUnit() );
+                    result = i18nc( "%1 is the temperature, %2 is the unit, like \"300 K\"", 
+                            "%1 %2", val, TempUnit::unitListSymbol( Prefs::temperatureUnit() ) );
                 }
                 break;
             }
@@ -170,7 +172,7 @@ QString KalziumUtils::prettyUnit( const Element* el, ChemicalDataObject::BlueObe
                 if ( val > 1600 ) {
                     result = i18n( "This element was discovered in the year <numid>%1</numid>.", val );
                 }else if( val == -1 ){
-                    result = i18n( "The element has not yet been offically recoqnized by the IUPAC." );
+                    result = i18n( "The element has not yet been officially recognized by the IUPAC." );
                 } //this should now really be 0. If not there is a bug in the database 
 		else {
                     result = i18n( "This element was known to ancient cultures." );
