@@ -15,8 +15,6 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 // 02110-1301, USA.
 
-#include <config.h>
-
 #include "latexexporter.h"
 
 #include "latexexporteroptions.h"
@@ -52,6 +50,8 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 
+#include <config-kig.h>
+
 #ifdef HAVE_TRUNC
 #define KDE_TRUNC(a)	trunc(a)
 #else
@@ -79,8 +79,7 @@ QString LatexExporter::menuEntryName() const
 
 QString LatexExporter::menuIcon() const
 {
-  // TODO
-  return "tex";
+  return "text-x-tex";
 }
 
 class LatexExportImpVisitor
@@ -94,12 +93,12 @@ class LatexExportImpVisitor
   QString mcurcolorid;
 public:
   void visit( ObjectHolder* obj );
-  void mapColor( QColor color );
+  void mapColor( const QColor& color );
 
   LatexExportImpVisitor( QTextStream& s, const KigWidget& w )
     : mstream( s ), mw( w ), msr( mw.showingRect() )
     {
-    };
+    }
   void visit( const LineImp* imp );
   void visit( const PointImp* imp );
   void visit( const TextImp* imp );
@@ -135,7 +134,7 @@ private:
    * Searches if a color is already mapped into mcolors, and returns its
    * index or -1 if not found.
    */
-  int findColor( QColor c );
+  int findColor( const QColor& c );
   /**
    * Use to convert a dimension "on the screen" to a dimension wrt.
    * Kig coordinate system.
@@ -177,7 +176,7 @@ void LatexExportImpVisitor::newLine()
   mstream << "\n";
 }
 
-int LatexExportImpVisitor::findColor( QColor c )
+int LatexExportImpVisitor::findColor( const QColor& c )
 {
   for ( uint i = 0; i < mcolors.size(); ++i )
   {
@@ -187,7 +186,7 @@ int LatexExportImpVisitor::findColor( QColor c )
   return -1;
 }
 
-void LatexExportImpVisitor::mapColor( QColor color )
+void LatexExportImpVisitor::mapColor( const QColor& color )
 {
   if ( findColor( color ) == -1 )
   {
@@ -483,31 +482,31 @@ void LatexExportImpVisitor::visit( const PolygonImp* imp )
 void LatexExporter::run( const KigPart& doc, KigWidget& w )
 {
   KigFileDialog* kfd = new KigFileDialog(
-      QString::null, i18n( "*.tex|Latex Documents (*.tex)" ),
+      QString(), i18n( "*.tex|Latex Documents (*.tex)" ),
       i18n( "Export as Latex" ), &w );
   kfd->setOptionCaption( i18n( "Latex Options" ) );
   LatexExporterOptions* opts = new LatexExporterOptions( 0L );
   kfd->setOptionsWidget( opts );
-  opts->showGridCheckBox->setChecked( doc.document().grid() );
-  opts->showAxesCheckBox->setChecked( doc.document().axes() );
-  opts->showExtraFrameCheckBox->setChecked( false );
+  opts->setGrid( doc.document().grid() );
+  opts->setAxes( doc.document().axes() );
+  opts->setExtraFrame( false );
   if ( !kfd->exec() )
     return;
 
   QString file_name = kfd->selectedFile();
-  bool showgrid = opts->showGridCheckBox->isOn();
-  bool showaxes = opts->showAxesCheckBox->isOn();
-  bool showframe = opts->showExtraFrameCheckBox->isOn();
+  bool showgrid = opts->showGrid();
+  bool showaxes = opts->showAxes();
+  bool showframe = opts->showExtraFrame();
 
   delete opts;
   delete kfd;
 
   QFile file( file_name );
-  if ( ! file.open( IO_WriteOnly ) )
+  if ( ! file.open( QIODevice::WriteOnly ) )
   {
     KMessageBox::sorry( &w, i18n( "The file \"%1\" could not be opened. Please "
-                                  "check if the file permissions are set correctly." )
-                        .arg( file_name ) );
+                                  "check if the file permissions are set correctly." ,
+                          file_name ) );
     return;
   };
 
