@@ -44,81 +44,78 @@
 #include <functional>
 
 #include <kaction.h>
-#include <kapplication.h>
 #include <kdebug.h>
+#include <kdemacros.h>
 #include <kfiledialog.h>
 #include <kglobal.h>
 #include <kiconloader.h>
-#include <kinstance.h>
+#include <kcomponentdata.h>
 #include <klocale.h>
-#include <kmainwindow.h>
+#include <kxmlguiwindow.h>
+#include <kactioncollection.h>
 #include <kmessagebox.h>
 #include <kmimetype.h>
-#include <kprinter.h>
 #include <kstandarddirs.h>
-#include <kstdaction.h>
-#include <ktoolbar.h>
+#include <kstandardaction.h>
+#include <ktoggleaction.h>
+#include <ktogglefullscreenaction.h>
+#include <kundostack.h>
 #include <kparts/genericfactory.h>
-#include <kdeprint/kprintdialogpage.h>
+#include <kicon.h>
+#include <kdeprintdialog.h>
+#include <kprintpreview.h>
 
+#include <qbytearray.h>
 #include <qcheckbox.h>
+#include <qeventloop.h>
 #include <qfile.h>
 #include <qlayout.h>
-#include <qpaintdevicemetrics.h>
 #include <qsizepolicy.h>
 #include <qtimer.h>
-#if QT_VERSION >= 0x030100
-#include <qeventloop.h>
-#endif
+#include <QtGui/QPrinter>
+#include <QtGui/QPrintDialog>
 
 using namespace std;
 
 static const QString typesFile = "macros.kigt";
 
 // export this library...
-typedef KParts::GenericFactory<KigPart> KigPartFactory;
-K_EXPORT_COMPONENT_FACTORY ( libkigpart, KigPartFactory )
-
-KAboutData* KigPart::createAboutData()
-{
-  return kigAboutData( "kig", I18N_NOOP( "KigPart" ) );
-}
-
-class SetCoordinateSystemAction
-  : public KSelectAction
-{
-  KigPart& md;
-public:
-  SetCoordinateSystemAction( KigPart& d, KActionCollection* parent );
-  void slotActivated( int index );
-};
+K_PLUGIN_FACTORY( KigPartFactory, registerPlugin< KigPart >(); )
+K_EXPORT_PLUGIN( KigPartFactory( kigAboutData( "kig", I18N_NOOP( "KigPart" ) ) ) )
 
 SetCoordinateSystemAction::SetCoordinateSystemAction(
   KigPart& d, KActionCollection* parent )
-  : KSelectAction( i18n( "&Set Coordinate System" ), 0, parent, "settings_set_coordinate_system" ),
+  : KSelectAction( i18n( "&Set Coordinate System" ), &d),
     md( d )
 {
   setItems( CoordinateSystemFactory::names() );
   setCurrentItem( md.document().coordinateSystem().id() );
+  connect( this, SIGNAL( triggered( int ) ), this, SLOT( slotActivated( int ) ) );
+  if(parent)
+    parent->addAction("settings_set_coordinate_system", this);
 }
 
 void SetCoordinateSystemAction::slotActivated( int index )
 {
   CoordinateSystem* sys = CoordinateSystemFactory::build( index );
   assert( sys );
-  md.history()->addCommand( KigCommand::changeCoordSystemCommand( md, sys ) );
+  md.history()->push( KigCommand::changeCoordSystemCommand( md, sys ) );
   setCurrentItem( index );
 }
 
 class KigPrintDialogPage
-  : public KPrintDialogPage
+  : public QWidget
 {
 public:
-  KigPrintDialogPage( QWidget* parent = 0, const char* name = 0 );
+  KigPrintDialogPage( QWidget* parent = 0 );
   ~KigPrintDialogPage();
 
-  void getOptions( QMap<QString,QString>& opts, bool );
-  void setOptions( const QMap<QString,QString>& opts );
+  bool printShowGrid();
+  void setPrintShowGrid( bool status );
+
+  bool printShowAxes();
+  void setPrintShowAxes( bool status );
+
   bool isValid( QString& );
 
 private:
@@ -126,40 +123,44 @@ private:
   QCheckBox *showaxes;
 };
 
-KigPrintDialogPage::KigPrintDialogPage( QWidget* parent, const char* name )
- : KPrintDialogPage( parent, name )
+KigPrintDialogPage::KigPrintDialogPage( QWidget* parent )
+ : QWidget( parent )
 {
-  setTitle( i18n( "Kig Options" ) );
+  setWindowTitle( i18n( "Kig Options" ) );
 
- QVBoxLayout* vl = new QVBoxLayout( this, 0 , 11 );
+  QVBoxLayout* vl = new QVBoxLayout( this );
 
- showgrid = new QCheckBox( i18n( "Show grid" ), this );
- vl->addWidget( showgrid );
+  showgrid = new QCheckBox( i18n( "Show grid" ), this );
+  vl->addWidget( showgrid );
 
- showaxes = new QCheckBox( i18n( "Show axes" ), this );
- vl->addWidget( showaxes );
+  showaxes = new QCheckBox( i18n( "Show axes" ), this );
+  vl->addWidget( showaxes );
 
- vl->addItem( new QSpacerItem( 10, 10, QSizePolicy::Fixed, QSizePolicy::Expanding ) );
+  vl->addItem( new QSpacerItem( 10, 10, QSizePolicy::Fixed, QSizePolicy::Expanding ) );
 }
 
 KigPrintDialogPage::~KigPrintDialogPage()
 {
 }
 
-void KigPrintDialogPage::getOptions( QMap< QString, QString >& opts, bool )
+bool KigPrintDialogPage::printShowGrid()
 {
-  opts[ "kde-kig-showgrid" ] = QString::number( showgrid->isChecked() );
-  opts[ "kde-kig-showaxes" ] = QString::number( showaxes->isChecked() );
+  return showgrid->isChecked();
 }
 
-void KigPrintDialogPage::setOptions( const QMap< QString, QString >& opts )
+void KigPrintDialogPage::setPrintShowGrid( bool status )
 {
-  QString tmp = opts[ "kde-kig-showgrid" ];
-  bool bt = ( tmp != "0" );
-  showgrid->setChecked( bt );
-  tmp = opts[ "kde-kig-showaxes" ];
-  bt = ( tmp != "0" );
-  showaxes->setChecked( bt );
+  showgrid->setChecked( status );
+}
+
+bool KigPrintDialogPage::printShowAxes()
+{
+  return showaxes->isChecked();
+}
+
+void KigPrintDialogPage::setPrintShowAxes( bool status )
+{
+  showaxes->setChecked( status );
 }
 
 bool KigPrintDialogPage::isValid( QString& )
@@ -167,19 +168,19 @@ bool KigPrintDialogPage::isValid( QString& )
   return true;
 }
 
-KigPart::KigPart( QWidget *parentWidget, const char *,
-			  QObject *parent, const char *name,
-			  const QStringList& )
-  : KParts::ReadWritePart( parent, name ),
-    mMode( 0 ), mdocument( new KigDocument() )
+KigPart::KigPart( QWidget *parentWidget, QObject *parent,
+                  const QVariantList& )
+  : KParts::ReadWritePart( parent ),
+    mMode( 0 ), mRememberConstruction( 0 ), mdocument( new KigDocument() )
 {
   // we need an instance
-  setInstance( KigPartFactory::instance() );
+  setComponentData( KigPartFactory::componentData() );
 
   mMode = new NormalMode( *this );
 
   // we need a widget, to actually show the document
-  m_widget = new KigView(this, false, parentWidget, "kig_view");
+  m_widget = new KigView( this, false, parentWidget );
+  m_widget->setObjectName( "kig_view" );
   // notify the part that this is our internal widget
   setWidget( m_widget );
 
@@ -193,9 +194,10 @@ KigPart::KigPart( QWidget *parentWidget, const char *,
   setupTypes();
 
   // construct our command history
-  mhistory = new KCommandHistory(actionCollection());
-  mhistory->documentSaved();
-  connect( mhistory, SIGNAL( documentRestored() ), this, SLOT( setUnmodified() ) );
+  mhistory = new KUndoStack();
+  mhistory->createUndoAction( actionCollection() );
+  mhistory->createRedoAction( actionCollection() );
+  connect( mhistory, SIGNAL( cleanChanged( bool ) ), this, SLOT( setHistoryClean( bool ) ) );
 
   // we are read-write by default
   setReadWrite(true);
@@ -208,122 +210,122 @@ KigPart::KigPart( QWidget *parentWidget, const char *,
 void KigPart::setupActions()
 {
   // save actions..
-  (void) KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
-  (void) KStdAction::save(this, SLOT(fileSave()), actionCollection());
+  (void) KStandardAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
+  (void) KStandardAction::save(this, SLOT(fileSave()), actionCollection());
 
   // print actions
-  (void) KStdAction::print( this, SLOT( filePrint() ), actionCollection() );
-  (void) KStdAction::printPreview( this, SLOT( filePrintPreview() ), actionCollection() );
+  (void) KStandardAction::print( this, SLOT( filePrint() ), actionCollection() );
+  (void) KStandardAction::printPreview( this, SLOT( filePrintPreview() ), actionCollection() );
 
   // selection actions
-  aSelectAll = KStdAction::selectAll(
+  aSelectAll = KStandardAction::selectAll(
     this, SLOT( slotSelectAll() ), actionCollection() );
-  aDeselectAll = KStdAction::deselect(
+  aDeselectAll = KStandardAction::deselect(
     this, SLOT( slotDeselectAll() ), actionCollection() );
-  aInvertSelection = new KAction(
-    i18n( "Invert Selection" ), "", 0, this,
-    SLOT( slotInvertSelection() ), actionCollection(),
-    "edit_invert_selection" );
+  aInvertSelection  = new KAction(i18n("Invert Selection"), this);
+  actionCollection()->addAction("edit_invert_selection", aInvertSelection );
+  connect(aInvertSelection, SIGNAL(triggered(bool) ), SLOT( slotInvertSelection() ));
 
   // we need icons...
-  KIconLoader* l = instance()->iconLoader();
-  QPixmap tmp;
+  KIconLoader* l = iconLoader();
 
-  aDeleteObjects = new KAction(
-      i18n("&Delete Objects"), "editdelete", Key_Delete, this,
-      SLOT(deleteObjects()), actionCollection(), "delete_objects");
+  aDeleteObjects  = new KAction(KIcon("edit-delete"), i18n("&Delete Objects"), this);
+  actionCollection()->addAction("delete_objects", aDeleteObjects );
+  connect(aDeleteObjects, SIGNAL(triggered(bool) ), SLOT(deleteObjects()));
+  aDeleteObjects->setShortcut(QKeySequence(Qt::Key_Delete));
   aDeleteObjects->setToolTip(i18n("Delete the selected objects"));
 
-  aCancelConstruction = new KAction(
-      i18n("Cancel Construction"), "stop", Key_Escape, this,
-      SLOT(cancelConstruction()), actionCollection(), "cancel_construction");
+  aCancelConstruction  = new KAction(KIcon("process-stop"), i18n("Cancel Construction"), this);
+  actionCollection()->addAction("cancel_construction", aCancelConstruction );
+  connect(aCancelConstruction, SIGNAL(triggered(bool) ), SLOT(cancelConstruction()));
+  aCancelConstruction->setShortcut(QKeySequence(Qt::Key_Escape));
   aCancelConstruction->setToolTip(
       i18n("Cancel the construction of the object being constructed"));
   aCancelConstruction->setEnabled(false);
 
-  aShowHidden = new KAction(
-    i18n("U&nhide All"), 0, this, SLOT( showHidden() ),
-    actionCollection(), "edit_unhide_all");
+  aRepeatLastConstruction = new KAction(KIcon("system-run"), i18n("Repeat Construction"), this);
+  actionCollection()->addAction("repeat_last_construction", aRepeatLastConstruction );
+  connect(aRepeatLastConstruction, SIGNAL(triggered(bool) ), SLOT(repeatLastConstruction()));
+  aRepeatLastConstruction->setShortcut(QKeySequence(Qt::Key_Z));
+  aRepeatLastConstruction->setToolTip(
+      i18n("Repeat the last construction (with new data)"));
+  aRepeatLastConstruction->setEnabled(false);
+
+  aShowHidden  = new KAction(i18n("U&nhide All"), this);
+  actionCollection()->addAction("edit_unhide_all", aShowHidden );
+  connect(aShowHidden, SIGNAL(triggered(bool) ), SLOT( showHidden() ));
   aShowHidden->setToolTip(i18n("Show all hidden objects"));
   aShowHidden->setEnabled( true );
 
-  aNewMacro = new KAction(
-    i18n("&New Macro..."), "gear", 0, this, SLOT(newMacro()),
-    actionCollection(), "macro_action");
+  aNewMacro  = new KAction(KIcon("system-run"), i18n("&New Macro..."), this);
+  actionCollection()->addAction("macro_action", aNewMacro );
+  connect(aNewMacro, SIGNAL(triggered(bool) ), SLOT(newMacro()));
   aNewMacro->setToolTip(i18n("Define a new macro"));
 
-  aConfigureTypes = new KAction(
-    i18n("Manage &Types..."), 0, this, SLOT(editTypes()),
-    actionCollection(), "types_edit");
+  aConfigureTypes  = new KAction(i18n("Manage &Types..."), this);
+  actionCollection()->addAction("types_edit", aConfigureTypes );
+  connect(aConfigureTypes, SIGNAL(triggered(bool) ), SLOT(editTypes()));
   aConfigureTypes->setToolTip(i18n("Manage macro types."));
 
+  aBrowseHistory  = new KAction(KIcon("view-history"), i18n("&Browse History..."), this);
+  actionCollection()->addAction("browse_history", aBrowseHistory );
+  connect( aBrowseHistory, SIGNAL( triggered( bool ) ), SLOT( browseHistory() ) );
+  aBrowseHistory->setToolTip( i18n( "Browse the history of the current construction." ) );
+
   KigExportManager::instance()->addMenuAction( this, m_widget->realWidget(),
                                                actionCollection() );
 
-  KAction* a = KStdAction::zoomIn( m_widget, SLOT( slotZoomIn() ),
+  QAction * a = KStandardAction::zoomIn( m_widget, SLOT( slotZoomIn() ),
                                    actionCollection() );
   a->setToolTip( i18n( "Zoom in on the document" ) );
   a->setWhatsThis( i18n( "Zoom in on the document" ) );
 
-  a = KStdAction::zoomOut( m_widget, SLOT( slotZoomOut() ),
+  a = KStandardAction::zoomOut( m_widget, SLOT( slotZoomOut() ),
                            actionCollection() );
   a->setToolTip( i18n( "Zoom out of the document" ) );
   a->setWhatsThis( i18n( "Zoom out of the document" ) );
 
-  a = KStdAction::fitToPage( m_widget, SLOT( slotRecenterScreen() ),
+  a = KStandardAction::fitToPage( m_widget, SLOT( slotRecenterScreen() ),
                              actionCollection() );
   // grr.. why isn't there an icon for this..
-  a->setIconSet( QIconSet( l->loadIcon( "view_fit_to_page", KIcon::Toolbar ) ) );
+  a->setIcon( KIcon( "view_fit_to_page", l ) );
   a->setToolTip( i18n( "Recenter the screen on the document" ) );
   a->setWhatsThis( i18n( "Recenter the screen on the document" ) );
 
-#ifdef KDE_IS_VERSION
-#if KDE_IS_VERSION(3,1,90)
-#define KIG_PART_CPP_STD_FULLSCREEN_ACTION
-#endif
-#endif
-#ifdef KIG_PART_CPP_STD_FULLSCREEN_ACTION
-  a = KStdAction::fullScreen( m_widget, SLOT( toggleFullScreen() ), actionCollection(), (QWidget*)(widget()->parent()),"fullscreen" );
-#else
-  tmp = l->loadIcon( "window_fullscreen", KIcon::Toolbar );
-  a = new KAction(
-    i18n( "Full Screen" ), tmp, CTRL+SHIFT+Key_F,
-    m_widget, SLOT( toggleFullScreen() ),
-    actionCollection(), "fullscreen" );
-#endif
+  a = actionCollection()->addAction(KStandardAction::FullScreen, "fullscreen", m_widget, SLOT( toggleFullScreen() ));
   a->setToolTip( i18n( "View this document full-screen." ) );
   a->setWhatsThis( i18n( "View this document full-screen." ) );
 
   // TODO: an icon for this..
-  a = new KAction(
-    i18n( "&Select Shown Area" ), "viewmagfit", 0, m_widget, SLOT( zoomRect() ),
-    actionCollection(), "view_select_shown_rect" );
+  a  = new KAction(KIcon("zoom-fit-best"), i18n("&Select Shown Area"), this);
+  actionCollection()->addAction("view_select_shown_rect", a );
+  connect(a, SIGNAL(triggered(bool) ), m_widget, SLOT( zoomRect() ));
   a->setToolTip( i18n( "Select the area that you want to be shown in the window." ) );
   a->setWhatsThis( i18n( "Select the area that you want to be shown in the window." ) );
 
-  a = new KAction(
-    i18n( "S&elect Zoom Area" ), "viewmag", 0, m_widget, SLOT( zoomArea() ),
-    actionCollection(), "view_zoom_area" );
+  a  = new KAction(KIcon("zoom-original"), i18n("S&elect Zoom Area"), this);
+  actionCollection()->addAction("view_zoom_area", a );
+  connect(a, SIGNAL(triggered(bool) ), m_widget, SLOT( zoomArea() ));
 //  a->setToolTip( i18n( "Select the area that you want to be shown in the window." ) );
 //  a->setWhatsThis( i18n( "Select the area that you want to be shown in the window." ) );
 
-  aToggleGrid = new KToggleAction(
-    i18n( "Show &Grid" ), 0, this, SLOT( toggleGrid() ),
-    actionCollection(), "settings_show_grid" );
+  aToggleGrid  = new KToggleAction(i18n("Show &Grid"), this);
+  actionCollection()->addAction("settings_show_grid", aToggleGrid );
   aToggleGrid->setToolTip( i18n( "Show or hide the grid." ) );
   aToggleGrid->setChecked( true );
+  connect( aToggleGrid, SIGNAL( triggered() ), this, SLOT( toggleGrid() ) );
 
-  aToggleAxes = new KToggleAction(
-    i18n( "Show &Axes" ), 0, this, SLOT( toggleAxes() ),
-    actionCollection(), "settings_show_axes" );
+  aToggleAxes  = new KToggleAction(i18n("Show &Axes"), this);
+  actionCollection()->addAction("settings_show_axes", aToggleAxes );
   aToggleAxes->setToolTip( i18n( "Show or hide the axes." ) );
   aToggleAxes->setChecked( true );
+  connect( aToggleAxes, SIGNAL( triggered() ), this, SLOT( toggleAxes() ) );
 
-  aToggleNightVision = new KToggleAction(
-    i18n( "Wear Infrared Glasses" ), 0, this, SLOT( toggleNightVision() ),
-    actionCollection(), "settings_toggle_nightvision" );
-  aToggleNightVision->setToolTip( i18n( "Enable/Disable hidden objects visibility." ) );
+  aToggleNightVision  = new KToggleAction(i18n("Wear Infrared Glasses"), this);
+  actionCollection()->addAction("settings_toggle_nightvision", aToggleNightVision );
+  aToggleNightVision->setToolTip( i18n( "Enable/disable hidden objects visibility." ) );
   aToggleNightVision->setChecked( false );
+  connect( aToggleNightVision, SIGNAL( triggered() ), this, SLOT( toggleNightVision() ) );
 
   // select coordinate system KActionMenu..
   aCoordSystem = new SetCoordinateSystemAction( *this, actionCollection() );
@@ -338,12 +340,26 @@ void KigPart::setupTypes()
   typedef GUIActionList::avectype::const_iterator iter;
   for ( iter i = l.actions().begin(); i != l.actions().end(); ++i )
   {
-    KigGUIAction* ret = new KigGUIAction( *i, *this, actionCollection() );
+    KigGUIAction* ret = new KigGUIAction( *i, *this );
     aActions.push_back( ret );
     ret->plug( this );
   };
 }
 
+void KigPart::rememberConstruction( ConstructibleAction* ca )
+{
+  // mp:
+  // mRememberConstruction holds a pointer to the last construction
+  // done by the user, so that it can be quickly repeated
+
+  mRememberConstruction = ca;
+  aRepeatLastConstruction->setEnabled(true);
+  aRepeatLastConstruction->setText( 
+      i18n( "Repeat Construction (%1)", ca->descriptiveName() ) );
+  aRepeatLastConstruction->setToolTip(
+      i18n( "Repeat %1 (with new data)", ca->descriptiveName() ) );
+}
+
 KigPart::~KigPart()
 {
   GUIActionList::instance()->unregDoc( this );
@@ -366,20 +382,23 @@ KigPart::~KigPart()
 
 bool KigPart::openFile()
 {
-  QFileInfo fileinfo( m_file );
+  QFileInfo fileinfo( localFilePath() );
   if ( ! fileinfo.exists() )
   {
     KMessageBox::sorry( widget(),
                         i18n( "The file \"%1\" you tried to open does not exist. "
-                              "Please verify that you entered the correct path." ).arg( m_file ),
+                              "Please verify that you entered the correct path.", localFilePath() ),
                         i18n( "File Not Found" ) );
     return false;
   };
 
-  // m_file is always local, so we can use findByPath instead of
-  // findByURL...
-  KMimeType::Ptr mimeType = KMimeType::findByPath ( m_file );
-  kdDebug() << k_funcinfo << "mimetype: " << mimeType->name() << endl;
+  KMimeType::Ptr mimeType = KMimeType::mimeType( arguments().mimeType() );
+  if ( !mimeType )
+  {
+    // we can always use findByPath instead of findByURL with localFilePath()
+    mimeType = KMimeType::findByPath( localFilePath() );
+  }
+  kDebug() << "mimetype: " << mimeType->name();
   KigFilter* filter = KigFilters::instance()->find( mimeType->name() );
   if ( !filter )
   {
@@ -390,16 +409,21 @@ bool KigPart::openFile()
         i18n( "You tried to open a document of type \"%1\"; unfortunately, "
               "Kig does not support this format. If you think the format in "
               "question would be worth implementing support for, you can "
-              "always ask us nicely on mailto:toscano.pino@tiscali.it "
+              "always ask us nicely on mailto:pino@kde.org "
               "or do the work yourself and send me a patch."
-          ).arg(mimeType->name()),
+          , mimeType->name()),
         i18n( "Format Not Supported" )
         );
     return false;
   };
 
-  KigDocument* newdoc = filter->load (m_file);
-  if ( !newdoc ) return false;
+  KigDocument* newdoc = filter->load (localFilePath());
+  if ( !newdoc )
+  {
+    closeUrl();
+    setUrl( KUrl() );
+    return false;
+  }
   delete mdocument;
   mdocument = newdoc;
   coordSystemChanged( mdocument->coordinateSystem().id() );
@@ -422,24 +446,24 @@ bool KigPart::openFile()
 
 bool KigPart::saveFile()
 {
-  if ( m_file.isEmpty() || m_bTemp ) return internalSaveAs();
+  if ( url().isEmpty() ) return internalSaveAs();
   // mimetype:
-  KMimeType::Ptr mimeType = KMimeType::findByPath ( m_file );
+  KMimeType::Ptr mimeType = KMimeType::findByPath ( localFilePath() );
   if ( mimeType->name() != "application/x-kig" )
   {
     // we don't support this mime type...
     if( KMessageBox::warningYesNo( widget(),
                      i18n( "Kig does not support saving to any other file format than "
                            "its own. Save to Kig's format instead?" ),
-                     i18n( "Format Not Supported" ), i18n("Save Kig Format"), KStdGuiItem::cancel() ) == KMessageBox::No )
+                     i18n( "Format Not Supported" ), KGuiItem( i18n( "Save Kig Format" ) ), KStandardGuiItem::cancel() ) == KMessageBox::No )
       return false;
     internalSaveAs();
   };
 
-  if ( KigFilters::instance()->save( document(), m_file ) )
+  if ( KigFilters::instance()->save( document(), localFilePath() ) )
   {
     setModified ( false );
-    mhistory->documentSaved();
+    mhistory->setClean();
     return true;
   }
   return false;
@@ -447,12 +471,12 @@ bool KigPart::saveFile()
 
 void KigPart::addObject(ObjectHolder* o)
 {
-  mhistory->addCommand( KigCommand::addCommand( *this, o ) );
+  mhistory->push( KigCommand::addCommand( *this, o ) );
 }
 
 void KigPart::addObjects( const std::vector<ObjectHolder*>& os )
 {
-  mhistory->addCommand( KigCommand::addCommand( *this, os ) );
+  mhistory->push( KigCommand::addCommand( *this, os ) );
 }
 
 void KigPart::_addObject( ObjectHolder* o )
@@ -504,6 +528,15 @@ void KigPart::cancelConstruction()
   mode()->cancelConstruction();
 }
 
+void KigPart::repeatLastConstruction()
+{
+  if ( mRememberConstruction )
+  {
+    ConstructibleAction* ca = mRememberConstruction;
+    ca->act( *this );
+  }
+}
+
 void KigPart::showHidden()
 {
   mode()->showHidden();
@@ -519,12 +552,17 @@ void KigPart::editTypes()
   mode()->editTypes();
 }
 
-void KigPart::setUnmodified()
+void KigPart::browseHistory()
 {
-  setModified( false );
+  mode()->browseHistory();
 }
 
-KCommandHistory* KigPart::history()
+void KigPart::setHistoryClean( bool clean )
+{
+  setModified( !clean );
+}
+
+QUndoStack* KigPart::history()
 {
   return mhistory;
 }
@@ -554,7 +592,7 @@ void KigPart::delObjects( const std::vector<ObjectHolder*>& os )
   assert( delobjs.size() >= os.size() );
 
   std::vector<ObjectHolder*> delobjsvect( delobjs.begin(), delobjs.end() );
-  mhistory->addCommand( KigCommand::removeCommand( *this, delobjsvect ) );
+  mhistory->push( KigCommand::removeCommand( *this, delobjsvect ) );
 }
 
 void KigPart::enableConstructActions( bool enabled )
@@ -603,26 +641,26 @@ void KigPart::fileSave()
 
 bool KigPart::internalSaveAs()
 {
-  // this slot is connected to the KStdAction::saveAs action...
+  // this slot is connected to the KStandardAction::saveAs action...
   QString formats = i18n( "*.kig|Kig Documents (*.kig)\n"
                           "*.kigz|Compressed Kig Documents (*.kigz)" );
 
   //  formats += "\n";
   //  formats += KImageIO::pattern( KImageIO::Writing );
 
-  QString file_name = KFileDialog::getSaveFileName(":document", formats );
+  QString file_name = KFileDialog::getSaveFileName( KUrl( "kfiledialog:///document" ), formats );
   if (file_name.isEmpty()) return false;
   else if ( QFileInfo( file_name ).exists() )
   {
     int ret = KMessageBox::warningContinueCancel( m_widget,
-                                         i18n( "The file \"%1\" already exists. Do you wish to overwrite it?" )
-                                         .arg( file_name ), i18n( "Overwrite File?" ), i18n("Overwrite") );
+                                         i18n( "The file \"%1\" already exists. Do you wish to overwrite it?" ,
+                                           file_name ), i18n( "Overwrite File?" ), KStandardGuiItem::overwrite() );
     if ( ret != KMessageBox::Continue )
     {
       return false;
     }
   }
-  saveAs(KURL::fromPathOrURL( file_name ));
+  saveAs( KUrl( file_name ) );
   return true;
 }
 
@@ -632,11 +670,9 @@ void KigPart::runMode( KigMode* m )
 
   setMode( m );
 
-#if QT_VERSION >= 0x030100
-  (void) kapp->eventLoop()->enterLoop();
-#else
-  (void) kapp->enter_loop();
-#endif
+  QEventLoop e;
+  m->setEventLoop( &e );
+  e.exec( QEventLoop::AllEvents | QEventLoop::WaitForMoreEvents );
 
   setMode( prev );
   redrawScreen();
@@ -645,13 +681,9 @@ void KigPart::runMode( KigMode* m )
 void KigPart::doneMode( KigMode* d )
 {
   assert( d == mMode );
-  // pretend to use this var..
-  (void)d;
-#if QT_VERSION >= 0x030100
-  kapp->eventLoop()->exitLoop();
-#else
-  kapp->exit_loop();
-#endif
+
+  if ( d->eventLoop() )
+    d->eventLoop()->exit();
 }
 
 void KigPart::actionRemoved( GUIAction* a, GUIUpdateToken& t )
@@ -667,19 +699,19 @@ void KigPart::actionRemoved( GUIAction* a, GUIUpdateToken& t )
     }
   };
   assert( rem );
-  aMNewSegment.remove( rem );
-  aMNewConic.remove( rem );
-  aMNewPoint.remove( rem );
-  aMNewCircle.remove( rem );
-  aMNewLine.remove( rem );
-  aMNewOther.remove( rem );
-  aMNewAll.remove( rem );
+  aMNewSegment.removeAll( rem );
+  aMNewConic.removeAll( rem );
+  aMNewPoint.removeAll( rem );
+  aMNewCircle.removeAll( rem );
+  aMNewLine.removeAll( rem );
+  aMNewOther.removeAll( rem );
+  aMNewAll.removeAll( rem );
   t.push_back( rem );
 }
 
 void KigPart::actionAdded( GUIAction* a, GUIUpdateToken& )
 {
-  KigGUIAction* ret = new KigGUIAction( a, *this, actionCollection() );
+  KigGUIAction* ret = new KigGUIAction( a, *this );
   aActions.push_back( ret );
   ret->plug( this );
 }
@@ -707,7 +739,7 @@ void KigPart::setupMacroTypes()
     // the user's saved macro types:
     QStringList dataFiles =
       KGlobal::dirs()->findAllResources("appdata", "kig-types/*.kigt",
-                                        true, false );
+                                        KStandardDirs::Recursive);
     std::vector<Macro*> macros;
     for ( QStringList::iterator file = dataFiles.begin();
           file != dataFiles.end(); ++file )
@@ -733,7 +765,7 @@ void KigPart::setupBuiltinMacros()
     // builtin macro types ( we try to make the user think these are
     // normal types )..
     QStringList builtinfiles =
-      KGlobal::dirs()->findAllResources( "appdata", "builtin-macros/*.kigt", true, false );
+      KGlobal::dirs()->findAllResources( "appdata", "builtin-macros/*.kigt", KStandardDirs::Recursive);
     for ( QStringList::iterator file = builtinfiles.begin();
           file != builtinfiles.end(); ++file )
     {
@@ -768,31 +800,34 @@ void KigPart::delWidget( KigWidget* v )
 
 void KigPart::filePrintPreview()
 {
-  KPrinter printer;
-  printer.setPreviewOnly( true );
-  doPrint( printer );
+  QPrinter printer;
+  KPrintPreview printPreview( &printer );
+  doPrint( printer, document().grid(), document().axes() );
+  printPreview.exec();
 }
 
 void KigPart::filePrint()
 {
-  KPrinter printer;
+  QPrinter printer;
   KigPrintDialogPage* kp = new KigPrintDialogPage();
-  printer.addDialogPage( kp );
+  QPrintDialog *printDialog = KdePrint::createPrintDialog( &printer, QList<QWidget*>() << kp, m_widget );
+  printDialog->setWindowTitle( i18n("Print Geometry") );
   printer.setFullPage( true );
-  printer.setOption( "kde-kig-showgrid", QString::number( document().grid() ) );
-  printer.setOption( "kde-kig-showaxes", QString::number( document().axes() ) );
-  printer.setPageSelection( KPrinter::ApplicationSide );
-  if ( printer.setup( m_widget, i18n("Print Geometry") ) )
+  //Unsupported in Qt
+  //printer.setPageSelection( QPrinter::ApplicationSide );
+  kp->setPrintShowGrid( document().grid() );
+  kp->setPrintShowAxes( document().axes() );
+  if (printDialog->exec())
   {
-    doPrint( printer );
-  };
+    doPrint( printer, kp->printShowGrid(), kp->printShowAxes() );
+  }
+  delete printDialog;
 }
 
-void KigPart::doPrint( KPrinter& printer )
+void KigPart::doPrint( QPrinter& printer, bool printGrid, bool printAxes )
 {
-  QPaintDeviceMetrics metrics( &printer );
   Rect rect = document().suggestedRect();
-  QRect qrect( 0, 0, metrics.width(), metrics.height() );
+  QRect qrect( 0, 0, printer.width(), printer.height() );
   if ( rect.width() * qrect.height() > rect.height() * qrect.width() )
   {
     // qrect is too high..
@@ -812,19 +847,7 @@ void KigPart::doPrint( KPrinter& printer )
   ScreenInfo si( rect, qrect );
   KigPainter painter( si, &printer, document() );
   painter.setWholeWinOverlay();
-  bool sg = true;
-  bool sa = true;
-  if ( !printer.previewOnly() )
-  {
-    sg = ( printer.option( "kde-kig-showgrid" ) != "0" );
-    sa = ( printer.option( "kde-kig-showaxes" ) != "0" );
-  }
-  else
-  {
-    sg = document().grid();
-    sg = document().axes();
-  }
-  painter.drawGrid( document().coordinateSystem(), sg, sa );
+  painter.drawGrid( document().coordinateSystem(), printGrid, printAxes );
   painter.drawObjects( document().objects(), false );
 }
 
@@ -855,11 +878,11 @@ void KigPart::hideObjects( const std::vector<ObjectHolder*>& inos )
   if ( os.size() == 0 ) return;
   else if ( os.size() == 1 )
     kc = new KigCommand( *this, os[0]->imp()->type()->hideAStatement() );
-  else kc = new KigCommand( *this, i18n( "Hide %n Object", "Hide %n Objects", os.size() ) );
+  else kc = new KigCommand( *this, i18np( "Hide %1 Object", "Hide %1 Objects", os.size() ) );
   for ( std::vector<ObjectHolder*>::iterator i = os.begin();
         i != os.end(); ++i )
     kc->addTask( new ChangeObjectDrawerTask( *i, ( *i )->drawer()->getCopyShown( false ) ) );
-  mhistory->addCommand( kc );
+  mhistory->push( kc );
 }
 
 void KigPart::showObjects( const std::vector<ObjectHolder*>& inos )
@@ -874,11 +897,11 @@ void KigPart::showObjects( const std::vector<ObjectHolder*>& inos )
   if ( os.size() == 0 ) return;
   else if ( os.size() == 1 )
     kc = new KigCommand( *this, os[0]->imp()->type()->showAStatement() );
-  else kc = new KigCommand( *this, i18n( "Show %n Object", "Show %n Objects", os.size() ) );
+  else kc = new KigCommand( *this, i18np( "Show %1 Object", "Show %1 Objects", os.size() ) );
   for ( std::vector<ObjectHolder*>::iterator i = os.begin();
         i != os.end(); ++i )
     kc->addTask( new ChangeObjectDrawerTask( *i, ( *i )->drawer()->getCopyShown( true ) ) );
-  mhistory->addCommand( kc );
+  mhistory->push( kc );
 }
 
 void KigPart::redrawScreen( KigWidget* w )
@@ -905,14 +928,14 @@ KigDocument& KigPart::document()
   return *mdocument;
 }
 
-extern "C" int convertToNative( const KURL& url, const QCString& outfile )
+extern "C" KDE_EXPORT int convertToNative( const KUrl& url, const QByteArray& outfile )
 {
-  kdDebug() << "converting " << url.prettyURL() << " to " << outfile << endl;
+  kDebug() << "converting " << url.prettyUrl() << " to " << outfile;
 
   if ( ! url.isLocalFile() )
   {
     // TODO
-    kdError() << "--convert-to-native only supports local files for now." << endl;
+    kError() << "--convert-to-native only supports local files for now." << endl;
     return -1;
   }
 
@@ -921,23 +944,23 @@ extern "C" int convertToNative( const KURL& url, const QCString& outfile )
   QFileInfo fileinfo( file );
   if ( ! fileinfo.exists() )
   {
-    kdError() << "The file \"" << file << "\" does not exist" << endl;
+    kError() << "The file \"" << file << "\" does not exist" << endl;
     return -1;
   };
 
   KMimeType::Ptr mimeType = KMimeType::findByPath ( file );
-  kdDebug() << k_funcinfo << "mimetype: " << mimeType->name() << endl;
+  kDebug() << "mimetype: " << mimeType->name();
   KigFilter* filter = KigFilters::instance()->find( mimeType->name() );
   if ( !filter )
   {
-    kdError() << "The file \"" << file << "\" is of a filetype not currently supported by Kig." << endl;
+    kError() << "The file \"" << file << "\" is of a filetype not currently supported by Kig." << endl;
     return -1;
   };
 
   KigDocument* doc = filter->load (file);
   if ( !doc )
   {
-    kdError() << "Parse error in file \"" << file << "\"." << endl;
+    kError() << "Parse error in file \"" << file << "\"." << endl;
     return -1;
   }
 
@@ -947,11 +970,11 @@ extern "C" int convertToNative( const KURL& url, const QCString& outfile )
   for ( std::vector<ObjectCalcer*>::iterator i = tmp.begin(); i != tmp.end(); ++i )
     ( *i )->calc( *doc );
 
-  QString out = ( outfile == "-" ) ? QString::null : outfile;
+  QString out = ( outfile == "-" ) ? QString() : outfile;
   bool success = KigFilters::instance()->save( *doc, out );
   if ( !success )
   {
-    kdError() << "something went wrong while saving" << endl;
+    kError() << "something went wrong while saving" << endl;
     return -1;
   }
 
