@@ -14,11 +14,11 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  USA
 **/
 
-#include <config.h>
+#include <math.h>
 
 #include "conic-common.h"
 
@@ -28,6 +28,8 @@
 #include <cmath>
 #include <algorithm>
 
+#include <config-kig.h>
+
 #ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
 #endif
