@@ -15,6 +15,8 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 // 02110-1301, USA.
 
+#include <math.h>
+
 #include "special_constructors.h"
 
 #include "calcpaths.h"
@@ -24,7 +26,9 @@
 #include "kigpainter.h"
 
 #include "../kig/kig_part.h"
+#include "../kig/kig_document.h"
 #include "../modes/construct_mode.h"
+#include "../objects/special_imptypes.h"
 #include "../objects/bogus_imp.h"
 #include "../objects/centerofcurvature_type.h"
 #include "../objects/circle_imp.h"
@@ -59,6 +63,227 @@
 #include <algorithm>
 #include <functional>
 
+/*
+ * conic-line and circle-circle intersection (with search for already computed
+ * intersections)
+ * the previous "ConicLineIntersectionConstructor" is now
+ * dead code, which could be remove in the future
+ */
+
+TwoOrOneIntersectionConstructor::TwoOrOneIntersectionConstructor(
+                                       const ArgsParserObjectType* t_std,
+                                       const ArgsParserObjectType* t_special,
+                                       const char* iconfile,
+                                       const struct ArgsParser::spec argsspecv[] )
+  : StandardConstructorBase( "SHOULD NOT BE SEEN", "SHOULD NOT BE SEEN",
+       iconfile, margsparser ),
+    mtype_std( t_std ),
+    mtype_special( t_special ),
+    margsparser( argsspecv, 2 )
+{
+}
+
+TwoOrOneIntersectionConstructor::~TwoOrOneIntersectionConstructor()
+{
+}
+
+void TwoOrOneIntersectionConstructor::drawprelim(
+           const ObjectDrawer& drawer, 
+           KigPainter& p, 
+           const std::vector<ObjectCalcer*>& parents,
+           const KigDocument& doc ) const
+{
+  Args args;
+  if ( parents.size() != 2 ) return;
+  transform( parents.begin(), parents.end(),
+             back_inserter( args ), std::mem_fun( &ObjectCalcer::imp ) );
+
+  for ( int i = -1; i <= 1; i += 2 )
+  {
+    IntImp param( i );
+    args.push_back( &param );
+    ObjectImp* data = mtype_std->calc( args, doc );
+    drawer.draw( *data, p, true );
+    delete data;
+    args.pop_back();
+  }
+}
+
+std::vector<ObjectCalcer*> removeDuplicatedPoints( std::vector<ObjectCalcer*> points )
+{
+  std::vector<ObjectCalcer*> ret;
+
+  for ( std::vector<ObjectCalcer*>::iterator i = points.begin();
+        i != points.end(); ++i )
+  {
+    for ( std::vector<ObjectCalcer*>::iterator j = ret.begin();
+          j != ret.end(); ++j )
+    {
+      if ( coincidentPoints( (*i)->imp(), (*j)->imp() ) ) break;
+    }
+    ret.push_back( *i );
+  }
+  return ret;
+}
+
+bool coincidentPoints( const ObjectImp* p1, const ObjectImp* p2 )
+{
+  const PointImp* pt1 = dynamic_cast<const PointImp*>( p1 );
+  if ( !pt1 ) return false;
+  const PointImp* pt2 = dynamic_cast<const PointImp*>( p2 );
+  if ( !pt2 ) return false;
+
+  Coordinate diff = pt1->coordinate() - pt2->coordinate();
+  if ( diff.squareLength() < 1e-12 ) return true;
+  return false;
+}
+
+std::vector<ObjectHolder*> TwoOrOneIntersectionConstructor::build( 
+           const std::vector<ObjectCalcer*>& parents, 
+           KigDocument& doc, 
+           KigWidget& ) const
+{
+  std::vector<ObjectHolder*> ret;
+  assert( parents.size() == 2 );
+
+  std::vector<ObjectCalcer*> points = doc.findIntersectionPoints( parents[0], parents[1] );
+  std::vector<ObjectCalcer*> uniquepoints = removeDuplicatedPoints( points );
+
+  if ( uniquepoints.size() == 1 )
+  {
+    std::vector<ObjectCalcer*> args( parents );
+    args.push_back( uniquepoints[0] );
+    ret.push_back( new ObjectHolder( new ObjectTypeCalcer(
+      mtype_special, args
+      ) ) );
+    return ret;
+  }
+  for ( int i = -1; i <= 1; i += 2 )
+  {
+    ObjectConstCalcer* d = new ObjectConstCalcer( new IntImp( i ) );
+    std::vector<ObjectCalcer*> args( parents );
+    args.push_back( d );
+
+    ret.push_back( new ObjectHolder( new ObjectTypeCalcer(
+      mtype_std, args
+      ) ) );
+  }
+  return ret;
+}
+
+void TwoOrOneIntersectionConstructor::plug( KigPart*, KigGUIAction* )
+{
+}
+
+bool TwoOrOneIntersectionConstructor::isTransform() const
+{
+  return false;
+}
+
+ThreeTwoOneIntersectionConstructor::ThreeTwoOneIntersectionConstructor(
+                                       const ArgsParserObjectType* t_std,
+                                       const ArgsParserObjectType* t_special,
+				       const ArgsParserObjectType* t_special2,
+                                       const char* iconfile,
+                                       const struct ArgsParser::spec argsspecv[] )
+  : StandardConstructorBase( "SHOULD NOT BE SEEN", "SHOULD NOT BE SEEN",
+       iconfile, margsparser ),
+    mtype_std( t_std ),
+    mtype_special( t_special ),
+    mtype_special2( t_special2 ),
+    margsparser( argsspecv, 2 )
+{
+}
+
+ThreeTwoOneIntersectionConstructor::~ThreeTwoOneIntersectionConstructor()
+{
+}
+
+void ThreeTwoOneIntersectionConstructor::drawprelim(
+           const ObjectDrawer& drawer, 
+           KigPainter& p, 
+           const std::vector<ObjectCalcer*>& parents,
+           const KigDocument& doc ) const
+{
+  Args args;
+  if ( parents.size() != 2 ) return;
+  transform( parents.begin(), parents.end(),
+             back_inserter( args ), std::mem_fun( &ObjectCalcer::imp ) );
+
+  for ( int i = 1; i <= 3; i += 1 )
+  {
+    IntImp param( i );
+    args.push_back( &param );
+    ObjectImp* data = mtype_std->calc( args, doc );
+    drawer.draw( *data, p, true );
+    delete data;
+    args.pop_back();
+  }
+}
+
+std::vector<ObjectHolder*> ThreeTwoOneIntersectionConstructor::build( 
+           const std::vector<ObjectCalcer*>& parents, 
+           KigDocument& doc, 
+           KigWidget& ) const
+{
+  std::vector<ObjectHolder*> ret;
+  assert( parents.size() == 2 );
+
+  std::vector<ObjectCalcer*> points = doc.findIntersectionPoints( parents[0], parents[1] );
+  std::vector<ObjectCalcer*> uniquepoints = removeDuplicatedPoints( points );
+
+  if ( uniquepoints.size() == 2 )
+  {
+    std::vector<ObjectCalcer*> args( parents );
+    args.push_back( uniquepoints[0] );
+    args.push_back( uniquepoints[1] );
+    ret.push_back( new ObjectHolder( new ObjectTypeCalcer(
+      mtype_special, args
+      ) ) );
+    return ret;
+  }
+  if ( uniquepoints.size() == 1 )
+  {
+  for ( int i = -1; i <= 1; i += 2 )
+  {
+     std::vector<ObjectCalcer*> args( parents );
+     args.push_back( uniquepoints[0] );
+     ObjectConstCalcer* d = new ObjectConstCalcer( new IntImp( i ) );
+     args.push_back( d );
+ 
+     ret.push_back( new ObjectHolder( new ObjectTypeCalcer(
+     CubicLineTwoIntersectionType::instance(), args
+      ) ) );
+    args.clear();
+  }
+    return ret;
+  }
+  for ( int i = 1; i <= 3; i += 1 )
+  {
+    ObjectConstCalcer* d = new ObjectConstCalcer( new IntImp( i ) );
+    std::vector<ObjectCalcer*> args( parents );
+    args.push_back( d );
+
+    ret.push_back( new ObjectHolder( new ObjectTypeCalcer(
+      mtype_std, args
+      ) ) );
+  }
+  return ret;
+}
+
+void ThreeTwoOneIntersectionConstructor::plug( KigPart*, KigGUIAction* )
+{
+}
+
+bool ThreeTwoOneIntersectionConstructor::isTransform() const
+{
+  return false;
+}
+
+/*
+ * conic-conic intersection
+ */
+
 class ConicConicIntersectionConstructor
   : public StandardConstructorBase
 {
@@ -200,7 +425,7 @@ void LocusConstructor::drawprelim( const ObjectDrawer& drawer, KigPainter& p, co
   drawer.draw( limp, p, true );
 }
 
-const int LocusConstructor::wantArgs(
+int LocusConstructor::wantArgs(
  const std::vector<ObjectCalcer*>& os, const KigDocument&, const KigWidget&
  ) const
 {
@@ -294,21 +519,21 @@ const QString PolygonBNPTypeConstructor::description() const
   return i18n("Construct a polygon by giving its vertices");
 }
 
-const QCString PolygonBNPTypeConstructor::iconFileName( const bool ) const
+const QByteArray PolygonBNPTypeConstructor::iconFileName( const bool ) const
 {
   return "kig_polygon";
 }
 
-const bool PolygonBNPTypeConstructor::isAlreadySelectedOK(
+bool PolygonBNPTypeConstructor::isAlreadySelectedOK(
  const std::vector<ObjectCalcer*>& os, const int& pos ) const
 {
   if ( pos == 0 && os.size() >= 3 ) return true;
   return false;
 }
 
-const int PolygonBNPTypeConstructor::wantArgs( const std::vector<ObjectCalcer*>& os,
-                                             const KigDocument&,
-                                             const KigWidget& ) const
+int PolygonBNPTypeConstructor::wantArgs( const std::vector<ObjectCalcer*>& os,
+                                         const KigDocument&,
+                                         const KigWidget& ) const
 {
   int count=os.size() - 1;
 
@@ -578,20 +803,20 @@ const QString PolygonBCVConstructor::description() const
   return i18n("Construct a regular polygon with a given center and vertex");
 }
 
-const QCString PolygonBCVConstructor::iconFileName( const bool ) const
+const QByteArray PolygonBCVConstructor::iconFileName( const bool ) const
 {
   return "hexagonbcv";
 }
 
-const bool PolygonBCVConstructor::isAlreadySelectedOK(
+bool PolygonBCVConstructor::isAlreadySelectedOK(
  const std::vector<ObjectCalcer*>&, const int& ) const
 {
   return false;
 }
 
-const int PolygonBCVConstructor::wantArgs( const std::vector<ObjectCalcer*>& os,
-                                           const KigDocument&,
-                                           const KigWidget& ) const
+int PolygonBCVConstructor::wantArgs( const std::vector<ObjectCalcer*>& os,
+                                     const KigDocument&,
+                                     const KigWidget& ) const
 {
   if ( os.size() > 3 ) return ArgsParser::Invalid;
 
@@ -697,6 +922,7 @@ void PolygonBCVConstructor::handlePrelim(
       circle.draw( p );
     }
   }
+  delete_all( args.begin() + 2, args.end() );
 }
 
 std::vector<ObjectHolder*> PolygonBCVConstructor::build( const std::vector<ObjectCalcer*>& parents, KigDocument&, KigWidget& ) const
@@ -749,15 +975,11 @@ QString PolygonBCVConstructor::useText( const ObjectCalcer&, const std::vector<O
 
     if ( winding > 1 )
     {
-      QString result = QString(
-        i18n( "Adjust the number of sides (%1/%2)" )
-        ).arg( nsides ).arg( winding );
+      QString result = i18n( "Adjust the number of sides (%1/%2)", nsides, winding );
       return result;
     } else
     {
-      QString result = QString(
-        i18n( "Adjust the number of sides (%1)" )
-        ).arg( nsides );
+      QString result = i18n( "Adjust the number of sides (%1)", nsides );
       return result;
     }
     break;
@@ -1052,35 +1274,42 @@ InversionConstructor::InversionConstructor()
       "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
       "inversion" );
 
-  SimpleObjectTypeConstructor* lineobj =
+  SimpleObjectTypeConstructor* curveobj =
      new SimpleObjectTypeConstructor(
-      InvertLineType::instance(),
+      CircularInversionType::instance(),
       "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
       "inversion" );
 
-  SimpleObjectTypeConstructor* segmentobj =
-     new SimpleObjectTypeConstructor(
-      InvertSegmentType::instance(),
-      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
-      "inversion" );
+//  SimpleObjectTypeConstructor* lineobj =
+//     new SimpleObjectTypeConstructor(
+//      InvertLineType::instance(),
+//      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
+//      "inversion" );
+//
+//  SimpleObjectTypeConstructor* segmentobj =
+//     new SimpleObjectTypeConstructor(
+//      InvertSegmentType::instance(),
+//      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
+//      "inversion" );
+//
+//  SimpleObjectTypeConstructor* circleobj =
+//     new SimpleObjectTypeConstructor(
+//      InvertCircleType::instance(),
+//      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
+//      "inversion" );
+//
+//  SimpleObjectTypeConstructor* arcobj =
+//     new SimpleObjectTypeConstructor(
+//      InvertArcType::instance(),
+//      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
+//      "inversion" );
 
-  SimpleObjectTypeConstructor* circleobj =
-     new SimpleObjectTypeConstructor(
-      InvertCircleType::instance(),
-      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
-      "inversion" );
-
-  SimpleObjectTypeConstructor* arcobj =
-     new SimpleObjectTypeConstructor(
-      InvertArcType::instance(),
-      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
-      "inversion" );
-
-  merge( arcobj );
-  merge( circleobj );
+//  merge( arcobj );
+//  merge( circleobj );
+  merge( curveobj );
   merge( pointobj );
-  merge( segmentobj );
-  merge( lineobj );
+//  merge( segmentobj );
+//  merge( lineobj );
 }
 
 InversionConstructor::~InversionConstructor() {}
@@ -1108,12 +1337,12 @@ const QString MeasureTransportConstructor::description() const
   return i18n("Transport the measure of a segment or arc over a line or circle.");
 }
 
-const QCString MeasureTransportConstructor::iconFileName( const bool ) const
+const QByteArray MeasureTransportConstructor::iconFileName( const bool ) const
 {
   return "measuretransport";
 }
 
-const bool MeasureTransportConstructor::isAlreadySelectedOK(
+bool MeasureTransportConstructor::isAlreadySelectedOK(
  const std::vector<ObjectCalcer*>&, const int& ) const
 {
   return false;
@@ -1125,15 +1354,14 @@ const bool MeasureTransportConstructor::isAlreadySelectedOK(
  * to the user
  */
 
-const int MeasureTransportConstructor::wantArgs(
+int MeasureTransportConstructor::wantArgs(
                                 const std::vector<ObjectCalcer*>& os,
-                                const KigDocument&,
+                                const KigDocument& doc,
                                 const KigWidget& ) const
 {
   if ( os.size() == 0 ) return ArgsParser::Valid;
 
-  if ( ! os[0]->imp()->inherits( SegmentImp::stype() ) &&
-       ! os[0]->imp()->inherits( ArcImp::stype() ) )
+  if ( ! os[0]->imp()->inherits( &lengthimptypeinstance ) )
     return ArgsParser::Invalid;
 
   if ( os.size() == 1 ) return ArgsParser::Valid;
@@ -1141,16 +1369,29 @@ const int MeasureTransportConstructor::wantArgs(
   if ( ! os[1]->imp()->inherits( LineImp::stype() ) &&
        ! os[1]->imp()->inherits( CircleImp::stype() ) )
     return ArgsParser::Invalid;
+  const CurveImp* c = static_cast<const CurveImp*>( os[1]->imp() );
 
   if ( os.size() == 2 ) return ArgsParser::Valid;
 
   if ( ! os[2]->imp()->inherits( PointImp::stype() ) )
     return ArgsParser::Invalid;
+  const PointImp* p = static_cast<const PointImp*>( os[2]->imp() );
 
-  // we here use the "isPointOnCurve", which relies on
-  // "by construction" incidence, instead of a numerical
-  // check
-  if ( ! isPointOnCurve( os[2], os[1] ) )
+  // we have two choices:
+  // - using "isPointOnCurve" produces a "by construction" incidence
+  //   test.  This would be fine, but doesn't always work; e.g. if we
+  //   have two points A, B, the segment s = AB and we construct the
+  //   support line of the segment (property of segments), then kig
+  //   is not able to understand that A is "by construction" on the
+  //   constructed line.
+  //   Moreover there are problems when hovering the cursor over points
+  //   that are on both a segment and its support line.
+  //      if ( ! isPointOnCurve( os[2], os[1] ) )
+  // - using "containsPoint", which is actually the test performed
+  //   when calc-ing the TransportOfMeasure; the risk here is to
+  //   be able to select points that are only coincidentally on the line.
+
+  if ( ! c->containsPoint( p->coordinate(), doc ) )
     return ArgsParser::Invalid;
 
   if ( os.size() == 3 ) return ArgsParser::Complete;
@@ -1209,6 +1450,8 @@ QString MeasureTransportConstructor::useText( const ObjectCalcer& o,
     return i18n("Segment to transport");
   if ( o.imp()->inherits( ArcImp::stype() ) )
     return i18n("Arc to transport");
+  if ( o.imp()->inherits( NumericTextImp::stype() ) )
+    return i18n("Value to transport");
   if ( o.imp()->inherits( LineImp::stype() ) )
     return i18n("Transport a measure on this line");
   if ( o.imp()->inherits( CircleImp::stype() ) )
@@ -1261,6 +1504,37 @@ bool MeasureTransportConstructor::isTransform() const
  * Generic intersection
  */
 
+/*
+ * these two argsparser spec vectors are used for the special
+ * construction of conic-line and circle-circle constructions
+ */
+
+static const struct ArgsParser::spec argsspeccli[] =
+{
+  { ConicImp::stype(), I18N_NOOP( "Intersect with this conic" ),
+    "SHOULD NOT BE SEEN", true },
+  { AbstractLineImp::stype(), I18N_NOOP( "Intersect with this line" ),
+    "SHOULD NOT BE SEEN", true }
+};
+
+
+static const struct ArgsParser::spec argsspeccbli[] =
+{
+  { CubicImp::stype(), I18N_NOOP( "Intersect with this cubic" ),
+    "SHOULD NOT BE SEEN", true },
+  { AbstractLineImp::stype(), I18N_NOOP( "Intersect with this line" ),
+    "SHOULD NOT BE SEEN", true }
+};
+
+
+static const struct ArgsParser::spec argsspeccci[] =
+{
+  { CircleImp::stype(), I18N_NOOP( "Intersect with this circle" ),
+    "SHOULD NOT BE SEEN", true },
+  { CircleImp::stype(), I18N_NOOP( "Intersect with this circle" ),
+    "SHOULD NOT BE SEEN", true }
+};
+
 GenericIntersectionConstructor::GenericIntersectionConstructor()
   : MergeObjectConstructor(
     I18N_NOOP( "Intersect" ),
@@ -1278,25 +1552,38 @@ GenericIntersectionConstructor::GenericIntersectionConstructor()
       "curvelineintersection" );
 
   ObjectConstructor* lineconic =
-    new ConicLineIntersectionConstructor();
+//    new ConicLineIntersectionConstructor();
+    new TwoOrOneIntersectionConstructor(
+          ConicLineIntersectionType::instance(),
+          ConicLineOtherIntersectionType::instance(),
+          "curvelineintersection",
+          argsspeccli);
 
   ObjectConstructor* arcline =
     new ArcLineIntersectionConstructor();
 
-  MultiObjectTypeConstructor* linecubic =
-    new MultiObjectTypeConstructor(
-      LineCubicIntersectionType::instance(),
-      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
-      "curvelineintersection", 1, 2, 3 );
+  ObjectConstructor* linecubic =
+    new ThreeTwoOneIntersectionConstructor(
+          LineCubicIntersectionType::instance(),
+          CubicLineOtherIntersectionType::instance(),
+	  CubicLineTwoIntersectionType::instance(),
+          "curvelineintersection",
+          argsspeccbli);
 
   ObjectConstructor* conicconic =
     new ConicConicIntersectionConstructor();
 
-  MultiObjectTypeConstructor* circlecircle =
-    new MultiObjectTypeConstructor(
-      CircleCircleIntersectionType::instance(),
-      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
-      "circlecircleintersection", -1, 1 );
+//  MultiObjectTypeConstructor* circlecircle =
+//    new MultiObjectTypeConstructor(
+//      CircleCircleIntersectionType::instance(),
+//      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
+//      "circlecircleintersection", -1, 1 );
+  ObjectConstructor* circlecircle =
+    new TwoOrOneIntersectionConstructor(
+          CircleCircleIntersectionType::instance(),
+          CircleCircleOtherIntersectionType::instance(),
+          "circlecircleintersection",
+          argsspeccci);
 
   SimpleObjectTypeConstructor* polygonline =
     new SimpleObjectTypeConstructor(
@@ -1304,6 +1591,12 @@ GenericIntersectionConstructor::GenericIntersectionConstructor()
       "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
       "curvelineintersection" );
 
+  SimpleObjectTypeConstructor* polygonpolygon =
+    new SimpleObjectTypeConstructor(
+      PolygonPolygonIntersectionType::instance(),
+      "SHOULDNOTBESEEN", "SHOULDNOTBESEEN",
+      "curvelineintersection" );
+
   merge( lineline );
   merge( circlecircle );
   merge( lineconic );
@@ -1311,6 +1604,7 @@ GenericIntersectionConstructor::GenericIntersectionConstructor()
   merge( conicconic );
   merge( arcline );
   merge( polygonline );
+  merge( polygonpolygon );
 }
 
 GenericIntersectionConstructor::~GenericIntersectionConstructor()
@@ -1361,7 +1655,7 @@ QString GenericIntersectionConstructor::useText(
       break;
   }
 
-  return QString::null;
+  return QString();
 }
 
 static const ArgsParser::spec argsspecMidPointOfTwoPoints[] =
@@ -1402,9 +1696,9 @@ std::vector<ObjectHolder*> MidPointOfTwoPointsConstructor::build(
 {
   ObjectTypeCalcer* seg = new ObjectTypeCalcer( SegmentABType::instance(), os );
   seg->calc( d );
-  int index = seg->imp()->propertiesInternalNames().findIndex( "mid-point" );
-  assert( index != -1 );
-  ObjectPropertyCalcer* prop = new ObjectPropertyCalcer( seg, index );
+//  int index = seg->imp()->propertiesInternalNames().indexOf( "mid-point" );
+//  assert( index != -1 );
+  ObjectPropertyCalcer* prop = new ObjectPropertyCalcer( seg, "mid-point" );
   prop->calc( d );
   std::vector<ObjectHolder*> ret;
   ret.push_back( new ObjectHolder( prop ) );
@@ -1466,7 +1760,7 @@ BaseConstructMode* TestConstructor::constructMode( KigPart& doc )
   return new TestConstructMode( doc, mtype );
 }
 
-const int TestConstructor::wantArgs( const std::vector<ObjectCalcer*>& os,
+int TestConstructor::wantArgs( const std::vector<ObjectCalcer*>& os,
                                      const KigDocument& d, const KigWidget& v ) const
 {
   int ret = StandardConstructorBase::wantArgs( os, d, v );
@@ -1541,7 +1835,7 @@ QString TangentConstructor::useText(
   else if ( o.imp()->inherits( PointImp::stype() ) )
     return i18n( "Tangent at This Point" );
 //  else assert( false );
-  return QString::null;
+  return QString();
 }
 
 //QString TangentConstructor::selectStatement(
@@ -1603,7 +1897,7 @@ QString CocConstructor::useText(
     return i18n( "Center of Curvature of This Curve" );
   else if ( o.imp()->inherits( PointImp::stype() ) )
     return i18n( "Center of Curvature at This Point" );
-  return QString::null;
+  return QString();
 }
 
 bool relativePrimes( int n, int p )
