@@ -34,6 +34,7 @@
 #include "../objects/object_factory.h"
 #include "../objects/polygon_imp.h"
 #include "../objects/text_imp.h"
+#include "../objects/text_type.h"
 #include "../misc/lists.h"
 #include "../misc/argsparser.h"
 #include "../misc/kigpainter.h"
@@ -46,24 +47,21 @@
 #include <algorithm>
 #include <functional>
 
+#include <qaction.h>
 #include <qcursor.h>
 #include <qdialog.h>
+#include <qfont.h>
 #include <qpen.h>
 #include <qregexp.h>
 #include <qvalidator.h>
 
 #include <kaction.h>
 #include <kcolordialog.h>
-#include <kglobal.h>
-#include <kiconloader.h>
-#include <klocale.h>
-#if KDE_IS_VERSION( 3, 1, 90 )
+#include <kicon.h>
 #include <kinputdialog.h>
-#else
-#include <klineeditdlg.h>
-#endif
+#include <klocale.h>
 
-#include <config.h>
+#include <config-kig.h>
 
 using namespace std;
 
@@ -184,13 +182,12 @@ NormalModePopupObjects::NormalModePopupObjects( KigPart& part,
                                                 KigWidget& view,
                                                 NormalMode& mode,
                                                 const std::vector<ObjectHolder*>& objs,
-						const QPoint& plc )
-  : KPopupMenu( &view ), mplc( plc ), mpart( part ), mview( view ), mobjs( objs ),
+                                                const QPoint& plc )
+  : KMenu( &view ), mplc( plc ), mpart( part ), mview( view ), mobjs( objs ),
     mmode( mode ), monlylabels( false )
 {
   bool empty = objs.empty();
   bool single = objs.size() == 1;
-  connect( this, SIGNAL( activated( int ) ), this, SLOT( toplevelMenuSlot( int ) ) );
 
   QString title;
   if ( empty )
@@ -203,8 +200,8 @@ NormalModePopupObjects::NormalModePopupObjects( KigPart& part,
       title = objs[0]->imp()->type()->translatedName();
   }
   else
-    title = i18n( "%1 Objects" ).arg( objs.size() );
-  insertTitle( title, 1 );
+    title = i18n( "%1 Objects", objs.size() );
+  addTitle( title );
 
   if ( !empty )
   {
@@ -243,34 +240,6 @@ NormalModePopupObjects::NormalModePopupObjects( KigPart& part,
   mproviders.push_back( new ScriptActionsProvider() );
 #endif
 
-  for ( uint i = 0; i < NumberOfMenus; ++i )
-    mmenus[i] = new QPopupMenu( this );
-
-  connect( mmenus[TransformMenu], SIGNAL( activated( int ) ),
-           this, SLOT( transformMenuSlot( int ) ) );
-  connect( mmenus[TestMenu], SIGNAL( activated( int ) ),
-           this, SLOT( testMenuSlot( int ) ) );
-  connect( mmenus[ConstructMenu], SIGNAL( activated( int ) ),
-           this, SLOT( constructMenuSlot( int ) ) );
-  connect( mmenus[StartMenu], SIGNAL( activated( int ) ),
-           this, SLOT( startMenuSlot( int ) ) );
-  connect( mmenus[ShowMenu], SIGNAL( activated( int ) ),
-           this, SLOT( showMenuSlot( int ) ) );
-  connect( mmenus[SetColorMenu], SIGNAL( activated( int ) ),
-           this, SLOT( setColorMenuSlot( int ) ) );
-  connect( mmenus[SetSizeMenu], SIGNAL( activated( int ) ),
-           this, SLOT( setSizeMenuSlot( int ) ) );
-  connect( mmenus[SetStyleMenu], SIGNAL( activated( int ) ),
-           this, SLOT( setStyleMenuSlot( int ) ) );
-  connect( mmenus[SetCoordinateSystemMenu], SIGNAL( activated( int ) ),
-           this, SLOT( setCoordinateSystemMenuSlot( int ) ) );
-
-  for ( int i = 0; i <= NumberOfMenus; ++i )
-  {
-    int nextfree = 10;
-    for ( uint j = 0; j < mproviders.size(); ++j )
-      mproviders[j]->fillUpMenu( *this, i, nextfree );
-  };
   static const QString menunames[NumberOfMenus] =
     {
       i18n( "&Transform" ),
@@ -281,66 +250,63 @@ NormalModePopupObjects::NormalModePopupObjects( KigPart& part,
       i18n( "Set Co&lor" ),
       i18n( "Set &Pen Width" ),
       i18n( "Set St&yle" ),
-      QString::null,
+      QString(),
       i18n( "Set Coordinate S&ystem" )
     };
   static const QString menuicons[NumberOfMenus] =
     {
       "centralsymmetry",
       "test",
-      QString::null,
-      "launch",
-      "kig_text",
-      "color_fill",
+      QString(),
+      "system-run",
+      "kig_text", // <- draw-text... hope I manage to rename it in time
+      "format-fill-color",
 //      "colorize",
       "sizer",
-      "paintbrush",
-      QString::null,
-      QString::null
+      "draw-brush",
+      QString(),
+      QString()
     };
-  int index = 1;
+
+  // creating the menus and setting their title and icon
+  KIconLoader* l = part.iconLoader();
+  for ( uint i = 0; i < NumberOfMenus; ++i )
+  {
+    mmenus[i] = new QMenu( this );
+    if ( !menunames[i].isEmpty() )
+      mmenus[i]->setTitle( menunames[i] );
+    if ( !menuicons[i].isEmpty() )
+    {
+      mmenus[i]->setIcon( KIcon( menuicons[i], l ) );
+    }
+  }
+
+  connect( this, SIGNAL( triggered( QAction* ) ),
+           this, SLOT( toplevelMenuSlot( QAction* ) ) );
+
   for ( int i = 0; i < NumberOfMenus; ++i )
   {
-    if ( mmenus[i]->count() == 0 ) continue;
-    if ( menuicons[i].isNull() )
-      insertItem( menunames[i], mmenus[i], i, index++ );
-    else
-    {
-      KIconLoader* l = part.instance()->iconLoader();
-      QPixmap icon = l->loadIcon( menuicons[i], KIcon::Small, 22, KIcon::DefaultState, 0L, true );
-      insertItem( QIconSet( icon ), menunames[i], mmenus[i], i, index++ );
-    }
+    int nextfree = 10;
+    for ( uint j = 0; j < mproviders.size(); ++j )
+      mproviders[j]->fillUpMenu( *this, i, nextfree );
   };
+  for ( int i = 0; i < NumberOfMenus; ++i )
+  {
+    if ( mmenus[i]->actions().count() == 0 ) continue;
+    if ( i == ToplevelMenu ) continue;
+    addMenu( mmenus[i] );
+  };
+  addActions( mmenus[ToplevelMenu]->actions() );
 }
 
-void NormalModePopupObjects::testMenuSlot( int i )
+void NormalModePopupObjects::toplevelMenuSlot( QAction* act )
 {
-  activateAction( TestMenu, i );
-}
-
-void NormalModePopupObjects::transformMenuSlot( int i )
-{
-  activateAction( TransformMenu, i );
-}
-
-void NormalModePopupObjects::constructMenuSlot( int i )
-{
-  activateAction( ConstructMenu, i );
-}
-
-void NormalModePopupObjects::startMenuSlot( int i )
-{
-  activateAction( StartMenu, i );
-}
-
-void NormalModePopupObjects::showMenuSlot( int i )
-{
-  activateAction( ShowMenu, i );
-}
-
-void NormalModePopupObjects::toplevelMenuSlot( int i )
-{
-  activateAction( ToplevelMenu, i );
+//  activateAction( ToplevelMenu, i );
+  int data = act->data().toInt();
+  int id = data & 0xFF;
+  int menu = data >> 8;
+kDebug() << "menu: " << menu << " - id: " << id;
+  activateAction( menu, id );
 }
 
 void NormalModePopupObjects::activateAction( int menu, int action )
@@ -349,6 +315,7 @@ void NormalModePopupObjects::activateAction( int menu, int action )
   // we need action - 10 cause we called fillUpMenu with nextfree set
   // to 10 initially..
   action -= 10;
+kDebug() << "MENU: " << menu << " - ACTION: " << action;
   for ( uint i = 0; ! done && i < mproviders.size(); ++i )
     done = mproviders[i]->executeAction( menu, action, mobjs, *this, mpart, mview, mmode );
 }
@@ -358,24 +325,30 @@ NormalModePopupObjects::~NormalModePopupObjects()
   delete_all ( mproviders.begin(), mproviders.end() );
 }
 
-static const QColor* colors[] =
+struct color_struct
 {
-  &Qt::blue,
-  &Qt::black,
-  &Qt::gray,
-  &Qt::red,
-  &Qt::green,
-  &Qt::cyan,
-  &Qt::yellow,
-  &Qt::darkRed
+  const Qt::GlobalColor color;
+  const char* name;
 };
-static const int numberofcolors = sizeof( colors ) / sizeof( QColor* );
+
+static const color_struct colors[] =
+{
+  { Qt::black, I18N_NOOP( "Black" ) },
+  { Qt::gray, I18N_NOOP( "Gray" ) },
+  { Qt::red, I18N_NOOP( "Red" ) },
+  { Qt::green, I18N_NOOP( "Green" ) },
+  { Qt::cyan, I18N_NOOP( "Cyan" ) },
+  { Qt::yellow, I18N_NOOP( "Yellow" ) },
+  { Qt::darkRed, I18N_NOOP( "Dark Red" ) }
+};
+
+const int numberofcolors = 7; // is there a better way to calc that?
 
 void BuiltinObjectActionsProvider::fillUpMenu( NormalModePopupObjects& popup, int menu, int& nextfree )
 {
+  KIconLoader* l = popup.part().iconLoader();
   if ( menu == NormalModePopupObjects::ToplevelMenu )
   {
-    KIconLoader* l = popup.part().instance()->iconLoader();
     std::vector<ObjectHolder*> os = popup.objects();
 
     /*
@@ -390,27 +363,25 @@ void BuiltinObjectActionsProvider::fillUpMenu( NormalModePopupObjects& popup, in
 
     if ( os.size() > 1 || os.front()->shown() )
     {
-      popup.addAction( menu, i18n( "&Hide" ), nextfree );
+      popup.addInternalAction( menu, i18n( "&Hide" ), nextfree );
     }
     if ( os.size() == 1 && !os.front()->shown() )
     {
-      popup.addAction( menu, i18n( "&Show" ), nextfree+1 );
+      popup.addInternalAction( menu, i18n( "&Show" ), nextfree+1 );
     }
     nextfree += 2;
-    QPixmap p = l->loadIcon( "move", KIcon::Toolbar );
-    popup.addAction( menu, p, i18n( "&Move" ), nextfree++ );
-    p = l->loadIcon( "editdelete", KIcon::Toolbar );
-    popup.addAction( menu, p, i18n( "&Delete" ), nextfree++ );
+    popup.addInternalAction( menu, KIcon( "transform-move", l ), i18n( "&Move" ), nextfree++ );
+    popup.addInternalAction( menu, KIcon( "edit-delete", l ), i18n( "&Delete" ), nextfree++ );
   }
   else if ( menu == NormalModePopupObjects::SetColorMenu )
   {
-    QPixmap p( 50, 20 );
-    for( const QColor** c = colors; c < colors + numberofcolors; ++c )
+    QPixmap p( 20, 20 );
+    for( int i = 0; i < numberofcolors; i++ )
     {
-      p.fill( **c );
-      popup.addAction( menu, p, nextfree++ );
+      p.fill( QColor( colors[i].color ) );
+      popup.addInternalAction( menu, QIcon( p ), i18n( colors[i].name ), nextfree++ );
     }
-    popup.addAction( menu, i18n( "&Custom Color" ), nextfree++ );
+    popup.addInternalAction( menu, KIcon( "colorize", l ), i18n( "&Custom Color" ), nextfree++ );
   }
   else if ( menu == NormalModePopupObjects::SetSizeMenu && !popup.onlyLabels() )
   {
@@ -425,10 +396,10 @@ void BuiltinObjectActionsProvider::fillUpMenu( NormalModePopupObjects& popup, in
       if ( (*i)->drawer()->color() != color ) samecolor = false;
     };
     if ( ! samecolor ) color = Qt::blue;
-    QPixmap p( point ? 20 : 50, 20 );
+    QPixmap p( 20, 20 );
     for ( int i = 1; i < 8; ++i )
     {
-      p.fill( popup.eraseColor() );
+      p.fill( popup.palette().color( popup.backgroundRole() ) );
       QPainter ptr( &p );
       ptr.setPen( QPen( color, 1 ) );
       ptr.setBrush( QBrush( color, Qt::SolidPattern ) );
@@ -444,7 +415,7 @@ void BuiltinObjectActionsProvider::fillUpMenu( NormalModePopupObjects& popup, in
         ptr.drawLine( QPoint( 0, 10 ), QPoint( 50, 10 ) );
       };
       ptr.end();
-      popup.addAction( menu, p, nextfree++ );
+      popup.addInternalAction( menu, QIcon( p ), nextfree++ );
     };
   }
   else if ( menu == NormalModePopupObjects::SetStyleMenu && !popup.onlyLabels() )
@@ -468,28 +439,28 @@ void BuiltinObjectActionsProvider::fillUpMenu( NormalModePopupObjects& popup, in
       for ( int i = 0; i < 5; ++i )
       {
         QPixmap p( 20, 20 );
-        p.fill( popup.eraseColor() );
+        p.fill( popup.palette().color( popup.backgroundRole() ) );
         ScreenInfo si( Rect( -1, -1, 2, 2 ), p.rect() );
         KigPainter ptr( si, &p, popup.part().document(), false );
         PointImp pt( Coordinate( 0, 0 ) );
         ObjectDrawer d( color, -1, true, Qt::SolidLine, i );
         d.draw( pt, ptr, false );
-        popup.addAction( menu, p, nextfree++ );
+        popup.addInternalAction( menu, QIcon( p ), nextfree++ );
       }
     else
     {
       Qt::PenStyle penstyles[] = {Qt::SolidLine, Qt::DashLine, Qt::DashDotLine, Qt::DashDotDotLine, Qt::DotLine};
       for ( int i = 0; i < (int) ( sizeof( penstyles ) / sizeof( Qt::PenStyle ) ); ++i )
       {
-        QPixmap p( 50, 20 );
-        p.fill( popup.eraseColor() );
-        ScreenInfo si( Rect( -2.5, -1, 5, 2 ), p.rect() );
+        QPixmap p( 20, 20 );
+        p.fill( popup.palette().color( popup.backgroundRole() ) );
+        ScreenInfo si( Rect( -2.5, -1, 2.5, 1 ), p.rect() );
         KigPainter ptr( si, &p, popup.part().document(), false );
         LineImp line( Coordinate( -1, 0 ), Coordinate( 1, 0 ) );
         Qt::PenStyle ps = penstyles[i];
-        ObjectDrawer d( color, -1, true, ps, 1 );
+        ObjectDrawer d( color, 2, true, ps, 1 );
         d.draw( line, ptr, false );
-        popup.addAction( menu, p, nextfree++ );
+        popup.addInternalAction( menu, QIcon( p ), nextfree++ );
       };
     }
   }
@@ -499,11 +470,11 @@ void NameObjectActionsProvider::fillUpMenu( NormalModePopupObjects& popup, int m
 {
   if ( menu == NormalModePopupObjects::ToplevelMenu )
   {
-    popup.addAction( menu, i18n( "Set &Name..." ), nextfree++ );
+    popup.addInternalAction( menu, i18n( "Set &Name..." ), nextfree++ );
   }
   else if ( menu == NormalModePopupObjects::ShowMenu )
   {
-    popup.addAction( menu, i18n( "&Name" ), nextfree++ );
+    popup.addInternalAction( menu, i18n( "&Name" ), nextfree++ );
   }
 }
 
@@ -536,15 +507,11 @@ bool NameObjectActionsProvider::executeAction(
     assert( os.size() == 1 );
     QString name = os[0]->name();
     bool ok;
-    QRegExp re(  ".*" );
-    QRegExpValidator* rev = new QRegExpValidator(  re,  &doc );
-    QString caption = i18n( "Set Object Name" );
-    QString label = i18n( "Set Name of this Object:" );
-#if KDE_IS_VERSION( 3, 1, 90 )
-    name = KInputDialog::getText( caption, label, name, &ok, &w, 0, rev );
-#else
-    name = KLineEditDlg::getText( caption, label, name, &ok, &w, rev );
-#endif
+    QRegExp re( ".*" );
+    QRegExpValidator* rev = new QRegExpValidator( re,  &doc );
+    name = KInputDialog::getText(
+               i18n( "Set Object Name" ), i18n( "Set Name of this Object:" ),
+               name, &ok, &w, rev );
     if ( ok )
     {
       bool justadded = false;
@@ -562,7 +529,7 @@ bool NameObjectActionsProvider::executeAction(
       cnamecalcer->setImp( new StringImp( name ) );
       KigCommand* kc = new KigCommand( doc, i18n( "Set Object Name" ) );
       mon.finish( kc );
-      doc.history()->addCommand( kc );
+      doc.history()->push( kc );
 
       // if we just added the name, we add a label to show it to the user.
       if ( justadded )
@@ -650,7 +617,7 @@ bool BuiltinObjectActionsProvider::executeAction(
     };
     QColor color;
     if ( id < numberofcolors )
-      color = *colors[id];
+      color = QColor( colors[id].color );
     else
     {
       if ( os.size() == 1 )
@@ -662,7 +629,7 @@ bool BuiltinObjectActionsProvider::executeAction(
     assert( color.isValid() );
     for ( std::vector<ObjectHolder*>::const_iterator i = os.begin(); i != os.end(); ++i )
       kc->addTask( new ChangeObjectDrawerTask( *i, ( *i )->drawer()->getCopyColor( color ) ) );
-    doc.history()->addCommand( kc );
+    doc.history()->push( kc );
     mode.clearSelection();
     return true;
   }
@@ -677,7 +644,7 @@ bool BuiltinObjectActionsProvider::executeAction(
     KigCommand* kc = new KigCommand( doc, i18n( "Change Object Width" ) );
     for ( std::vector<ObjectHolder*>::const_iterator i = os.begin(); i != os.end(); ++i )
       kc->addTask( new ChangeObjectDrawerTask( *i, ( *i )->drawer()->getCopyWidth( 1 + 2 * id ) ) );
-    doc.history()->addCommand( kc );
+    doc.history()->push( kc );
     mode.clearSelection();
     return true;
   }
@@ -706,7 +673,7 @@ bool BuiltinObjectActionsProvider::executeAction(
       for ( std::vector<ObjectHolder*>::const_iterator i = os.begin(); i != os.end(); ++i )
         if ( (*i)->imp()->inherits( PointImp::stype() ) )
           kc->addTask( new ChangeObjectDrawerTask( *i, ( *i )->drawer()->getCopyPointStyle( id ) ) );
-      doc.history()->addCommand( kc );
+      doc.history()->push( kc );
       mode.clearSelection();
       return true;
     }
@@ -719,7 +686,7 @@ bool BuiltinObjectActionsProvider::executeAction(
       for ( std::vector<ObjectHolder*>::const_iterator i = os.begin(); i != os.end(); ++i )
         if ( ! (*i)->imp()->inherits( PointImp::stype() ) )
           kc->addTask( new ChangeObjectDrawerTask( *i, ( *i )->drawer()->getCopyStyle( p ) ) );
-      doc.history()->addCommand( kc );
+      doc.history()->push( kc );
       mode.clearSelection();
     }
     return true;
@@ -753,14 +720,13 @@ void ObjectConstructorActionsProvider::fillUpMenu( NormalModePopupObjects& popup
     };
     if ( add )
     {
-      QCString iconfile = (*i)->iconFileName();
+      QByteArray iconfile = (*i)->iconFileName();
       if ( !iconfile.isEmpty() && !iconfile.isNull() )
       {
-        QPixmap icon = popup.part().instance()->iconLoader()->loadIcon( iconfile, KIcon::Toolbar, 22, KIcon::DefaultState, 0L, true );
-        popup.addAction( menu, icon, (*i)->descriptiveName(), nextfree++ );
+        popup.addInternalAction( menu, KIcon( iconfile, popup.part().iconLoader() ), (*i)->descriptiveName(), nextfree++ );
       }
       else
-        popup.addAction( menu, (*i)->descriptiveName(), nextfree++ );
+        popup.addInternalAction( menu, (*i)->descriptiveName(), nextfree++ );
       mctors[menu].push_back( *i );
     }
   };
@@ -794,57 +760,33 @@ bool ObjectConstructorActionsProvider::executeAction(
   return true;
 }
 
-void NormalModePopupObjects::addAction( int menu, const QPixmap& pix, int id )
+QAction* NormalModePopupObjects::addInternalAction( int menu, const QIcon& pix, int id )
 {
-  QPopupMenu* m = 0;
-  if ( menu == ToplevelMenu ) m = this;
-  else m = mmenus[menu];
-  int ret = m->insertItem( pix, id );
-  assert( ret == id );
-  // pretend to use this var..
-  (void) ret;
+  return addInternalAction( menu, pix, "", id );
 }
 
-void NormalModePopupObjects::setColorMenuSlot( int i )
+QAction* NormalModePopupObjects::addInternalAction( int menu, const QIcon& icon, const QString& name, int id )
 {
-  activateAction( SetColorMenu, i );
+//kDebug() << "ID: " << id;
+  QMenu* m = mmenus[menu];
+  QAction* newaction = m->addAction( icon, name );
+  newaction->setData( QVariant::fromValue( ( menu << 8 ) | id ) );
+  return newaction;
 }
 
-void NormalModePopupObjects::setSizeMenuSlot( int i )
+QAction* NormalModePopupObjects::addInternalAction( int menu, const QString& name, int id )
 {
-  activateAction( SetSizeMenu, i );
+  QMenu* m = mmenus[menu];
+  QAction* newaction = m->addAction( name );
+  newaction->setData( QVariant::fromValue( ( menu << 8 ) | id ) );
+  return newaction;
 }
 
-void NormalModePopupObjects::setStyleMenuSlot( int i )
+QAction* NormalModePopupObjects::addInternalAction( int menu, QAction* act )
 {
-  activateAction( SetStyleMenu, i );
-}
-
-void NormalModePopupObjects::setCoordinateSystemMenuSlot( int i )
-{
-  activateAction( SetCoordinateSystemMenu, i );
-}
-
-void NormalModePopupObjects::addAction( int menu, const QPixmap& icon, const QString& name, int id )
-{
-  QPopupMenu* m = 0;
-  if ( menu == ToplevelMenu ) m = this;
-  else m = mmenus[menu];
-  int ret = m->insertItem( QIconSet( icon ), name, id );
-  assert( ret == id );
-  // pretend to use this var..
-  (void)ret;
-}
-
-void NormalModePopupObjects::addAction( int menu, const QString& name, int id )
-{
-  QPopupMenu* m = 0;
-  if ( menu == ToplevelMenu ) m = this;
-  else m = mmenus[menu];
-  int ret = m->insertItem( name, id );
-  assert( ret == id );
-  // pretend to use this var..
-  (void)ret;
+  QMenu* m = mmenus[menu];
+  m->addAction( act );
+  return act;
 }
 
 PopupActionProvider::~PopupActionProvider()
@@ -856,10 +798,10 @@ void PropertiesActionsProvider::fillUpMenu( NormalModePopupObjects& popup,
 {
   if ( popup.objects().size() != 1 ) return;
   ObjectHolder* o = popup.objects()[0];
-  uint np = o->imp()->numberOfProperties();
+  int np = o->imp()->numberOfProperties();
   if ( menu != NormalModePopupObjects::ConstructMenu &&
        menu != NormalModePopupObjects::ShowMenu ) return;
-  for ( uint i = 0; i < np; ++i )
+  for ( int i = 0; i < np; ++i )
   {
     ObjectImp* prop = o->imp()->property( i, popup.part().document() );
     const char* iconfile = o->imp()->iconForProperty( i );
@@ -881,12 +823,11 @@ void PropertiesActionsProvider::fillUpMenu( NormalModePopupObjects& popup,
     {
       if ( iconfile && *iconfile )
       {
-        QPixmap pix = popup.part().instance()->iconLoader()->loadIcon( iconfile, KIcon::Toolbar, 22, KIcon::DefaultState, 0L, true );
-        popup.addAction( menu, pix, i18n( o->imp()->properties()[i] ), nextfree++ );
+        popup.addInternalAction( menu, KIcon( iconfile, popup.part().iconLoader() ), i18n( o->imp()->properties()[i] ), nextfree++ );
       }
       else
       {
-        popup.addAction( menu, i18n( o->imp()->properties()[i] ), nextfree++ );
+        popup.addInternalAction( menu, i18n( o->imp()->properties()[i] ), nextfree++ );
       };
       mprops[menu-1].push_back( i );
     };
@@ -913,7 +854,7 @@ bool PropertiesActionsProvider::executeAction(
   if ( menu == NormalModePopupObjects::ShowMenu )
   {
     std::vector<ObjectCalcer*> args;
-    args.push_back( new ObjectPropertyCalcer( parent->calcer(), propid ) );
+    args.push_back( new ObjectPropertyCalcer( parent->calcer(), propid, true ) );
     args.back()->calc( doc.document() );
 // TODO: recover the cursor position somehow... the following does not work
 // in general...
@@ -931,7 +872,7 @@ bool PropertiesActionsProvider::executeAction(
   else
   {
     ObjectHolder* h = new ObjectHolder(
-      new ObjectPropertyCalcer( parent->calcer(), propid ) );
+      new ObjectPropertyCalcer( parent->calcer(), propid, true ) );
     h->calc( doc.document() );
     doc.addObject( h );
   };
@@ -951,7 +892,7 @@ void ObjectTypeActionsProvider::fillUpMenu(
   QStringList l = t->specialActions();
   mnoa = l.count();
   for ( int i = 0; i < mnoa; ++i )
-    popup.addAction( menu, l[i], nextfree++ );
+    popup.addInternalAction( menu, l.at( i ), nextfree++ );
 }
 
 bool ObjectTypeActionsProvider::executeAction(
@@ -977,21 +918,25 @@ void BuiltinDocumentActionsProvider::fillUpMenu( NormalModePopupObjects& popup,
 {
   if ( menu == NormalModePopupObjects::ToplevelMenu )
   {
-    popup.addAction( menu, i18n( "U&nhide All" ), nextfree++ );
-    popup.part().action( "view_zoom_in" )->plug( &popup );
-    popup.part().action( "view_zoom_out" )->plug( &popup );
-    popup.part().action( "fullscreen" )->plug( &popup );
+    popup.addInternalAction( menu, i18n( "U&nhide All" ), nextfree++ );
+    popup.addInternalAction( menu, popup.part().action( "view_zoom_in" ) );
+    popup.addInternalAction( menu, popup.part().action( "view_zoom_out" ) );
+    popup.addInternalAction( menu, popup.part().action( "fullscreen" ) );
     nextfree += 3;
   }
   else if ( menu == NormalModePopupObjects::SetCoordinateSystemMenu )
   {
-    int idoffset = nextfree;
     QStringList l = CoordinateSystemFactory::names();
     mnumberofcoordsystems = l.count();
-    for ( uint i = 0; i < l.count(); ++i )
-      popup.addAction( menu, l[i], nextfree++ );
     int current = popup.part().document().coordinateSystem().id();
-    popup.setChecked( menu, idoffset + current, true );
+    QAction* act = 0;
+    for ( int i = 0; i < mnumberofcoordsystems; ++i )
+    {
+      act = popup.addInternalAction( menu, l.at( i ), nextfree++ );
+      act->setCheckable( true );
+      if ( i == current )
+        act->setChecked( true );
+    }
   }
 }
 
@@ -1002,7 +947,7 @@ bool BuiltinDocumentActionsProvider::executeAction(
 {
   if ( menu == NormalModePopupObjects::ToplevelMenu )
   {
-    kdDebug() << "id: " << id << endl;
+    kDebug() << "id: " << id;
     if ( id == 0 )
     {
       doc.showHidden();
@@ -1021,18 +966,13 @@ bool BuiltinDocumentActionsProvider::executeAction(
     };
     CoordinateSystem* sys = CoordinateSystemFactory::build( id );
     assert( sys );
-    doc.history()->addCommand( KigCommand::changeCoordSystemCommand( doc, sys ) );
+    doc.history()->push( KigCommand::changeCoordSystemCommand( doc, sys ) );
     m.clearSelection();
     return true;
   }
   else return false;
 }
 
-void NormalModePopupObjects::setChecked( int menu, int n, bool checked )
-{
-  mmenus[menu]->setItemChecked( n, checked );
-}
-
 #ifdef KIG_ENABLE_PYTHON_SCRIPTING
 /**
  * this is a local function that looks for a python script associated
@@ -1045,6 +985,19 @@ static ObjectTypeCalcer* getPythonExecuteTypeFromCalcer( ObjectCalcer* o )
   const PythonExecuteType* pythonexec = dynamic_cast<const PythonExecuteType*>( oc->type() );
   if ( pythonexec ) return oc;
 
+  const GenericTextType* text = dynamic_cast<const GenericTextType*>( oc->type() );
+  if ( !text ) return 0;
+  std::vector<ObjectCalcer*> parents = oc->parents();
+
+  for (uint i = 3; i < parents.size(); i++)
+  {
+    oc = dynamic_cast<ObjectTypeCalcer *>( parents[i] );
+    if ( oc )
+    {
+      pythonexec = dynamic_cast<const PythonExecuteType*>( oc->type() );
+      if ( pythonexec ) return oc;
+    }
+  }
   return 0;
 }
 
@@ -1052,9 +1005,8 @@ void ScriptActionsProvider::fillUpMenu( NormalModePopupObjects& popup, int menu,
 {
   if ( menu == NormalModePopupObjects::StartMenu )
   {
-    KIconLoader* l = popup.part().instance()->iconLoader();
-    QPixmap p = l->loadIcon( ScriptType::icon( ScriptType::Python ), KIcon::Toolbar, 22, KIcon::DefaultState, 0L, true );
-    popup.addAction( menu, p, i18n( "Python Script" ), nextfree++ );
+    KIconLoader* l = popup.part().iconLoader();
+    popup.addInternalAction( menu, KIcon( ScriptType::icon( ScriptType::Python ), l ), i18n( "Python Script" ), nextfree++ );
     mns++;
   }
   else if ( menu == NormalModePopupObjects::ToplevelMenu )
@@ -1062,7 +1014,7 @@ void ScriptActionsProvider::fillUpMenu( NormalModePopupObjects& popup, int menu,
     if ( !popup.objects().empty() &&
          getPythonExecuteTypeFromCalcer( popup.objects().front()->calcer() ) )
     {
-      popup.addAction( menu, i18n( "Edit Script..." ), nextfree );
+      popup.addInternalAction( menu, "Edit Script...", nextfree );
     }
     nextfree++;
   }
@@ -1167,7 +1119,7 @@ int ObjectChooserPopup::getObjectFromList( const QPoint& p, KigWidget* w,
       id = 1;
     }
   }
-//  kdDebug() << "numpoints: " << numpoints << endl
+//  kDebug() << "numpoints: " << numpoints
 //            << "numothers: " << numothers << endl
 //            << "numpolygons: " << numpolygons << endl
 //            << "id: " << id << endl;
@@ -1177,24 +1129,27 @@ int ObjectChooserPopup::getObjectFromList( const QPoint& p, KigWidget* w,
 
 ObjectChooserPopup::ObjectChooserPopup( const QPoint& p, KigWidget& view,
                                         const std::vector<ObjectHolder*>& objs )
-  : KPopupMenu(), mplc( p ), mview( view ), mobjs( objs ), mselected( -1 )
+  : KMenu(), mplc( p ), mview( view ), mobjs( objs ), mselected( -1 )
 {
+  addTitle( i18np( "%1 Object", "%1 Objects", mobjs.size() ) );
+  QAction* newaction = 0;
   for ( uint i = 0; i < mobjs.size(); i++ )
   {
-    insertItem( !mobjs[i]->name().isEmpty()
+    newaction = addAction(
+                !mobjs[i]->name().isEmpty()
                 ? QString::fromLatin1( "%1 %2" ).arg( mobjs[i]->imp()->type()->translatedName() ).arg( mobjs[i]->name() )
-                : mobjs[i]->imp()->type()->translatedName(),
-                i );
+                : mobjs[i]->imp()->type()->translatedName() );
+    newaction->setData( QVariant::fromValue( i ) );
   }
 
-  connect( this, SIGNAL( activated( int ) ), this, SLOT( actionActivatedSlot( int ) ) );
+  connect( this, SIGNAL( triggered( QAction* ) ), this, SLOT( actionActivatedSlot( QAction* ) ) );
 }
 
 ObjectChooserPopup::~ObjectChooserPopup()
 {
 }
 
-void ObjectChooserPopup::actionActivatedSlot( int which )
+void ObjectChooserPopup::actionActivatedSlot( QAction* act )
 {
-  mselected = which;
+  mselected = act->data().toInt();
 }
