@@ -1,4 +1,3 @@
-// locus_imp.h
 // Copyright (C)  2003  Dominique Devriese <devriese@kde.org>
 
 // This program is free software; you can redistribute it and/or
@@ -13,8 +12,8 @@
 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-// 02111-1307, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+// 02110-1301, USA.
 
 #ifndef KIG_OBJECTS_LOCUS_IMP_H
 #define KIG_OBJECTS_LOCUS_IMP_H
@@ -42,10 +41,10 @@
  * much, and because every ObjectImp should contain all the data it
  * needs itself.  ObjectImp's are entirely independent objects.
  * That's also why we don't keep a pointer to the old CurveImp, but a
- * copy of it..
+ * copy of it.
  *
  * i hope this is a bit clear, if not, feel free to ask for
- * explanation of what you don't understand..
+ * explanation of what you don't understand.
  */
 class LocusImp
   : public CurveImp
@@ -74,13 +73,13 @@ public:
   const Coordinate getPoint( double param, const KigDocument& ) const;
 
   // TODO ?
-  const uint numberOfProperties() const;
-  const QCStringList properties() const;
-  const QCStringList propertiesInternalNames() const;
-  ObjectImp* property( uint which, const KigDocument& w ) const;
-  const char* iconForProperty( uint which ) const;
-  const ObjectImpType* impRequirementForProperty( uint which ) const;
-  bool isPropertyDefinedOnOrThroughThisImp( uint which ) const;
+  int numberOfProperties() const;
+  const QByteArrayList properties() const;
+  const QByteArrayList propertiesInternalNames() const;
+  ObjectImp* property( int which, const KigDocument& w ) const;
+  const char* iconForProperty( int which ) const;
+  const ObjectImpType* impRequirementForProperty( int which ) const;
+  bool isPropertyDefinedOnOrThroughThisImp( int which ) const;
 
   const CurveImp* curve() const;
   const ObjectHierarchy& hierarchy() const;
