@@ -15,11 +15,14 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 // 02110-1301, USA.
 
+#include <math.h>
+
 #include "other_imp.h"
 
 #include "bogus_imp.h"
 #include "point_imp.h"
 #include "line_imp.h"
+#include "circle_imp.h"
 
 #include "../misc/screeninfo.h"
 #include "../misc/common.h"
@@ -81,14 +84,14 @@ Coordinate AngleImp::attachPoint() const
   return mpoint;
 }
 
-const uint AngleImp::numberOfProperties() const
+int AngleImp::numberOfProperties() const
 {
   return Parent::numberOfProperties() + 3;
 }
 
-const QCStringList AngleImp::propertiesInternalNames() const
+const QByteArrayList AngleImp::propertiesInternalNames() const
 {
-  QCStringList l = Parent::propertiesInternalNames();
+  QByteArrayList l = Parent::propertiesInternalNames();
   l << "angle-radian";
   l << "angle-degrees";
   l << "angle-bisector";
@@ -96,9 +99,9 @@ const QCStringList AngleImp::propertiesInternalNames() const
   return l;
 }
 
-const QCStringList AngleImp::properties() const
+const QByteArrayList AngleImp::properties() const
 {
-  QCStringList l = Parent::properties();
+  QByteArrayList l = Parent::properties();
   l << I18N_NOOP( "Angle in Radians" );
   l << I18N_NOOP( "Angle in Degrees" );
   l << I18N_NOOP( "Angle Bisector" );
@@ -106,14 +109,14 @@ const QCStringList AngleImp::properties() const
   return l;
 }
 
-const ObjectImpType* AngleImp::impRequirementForProperty( uint which ) const
+const ObjectImpType* AngleImp::impRequirementForProperty( int which ) const
 {
   if ( which < Parent::numberOfProperties() )
     return Parent::impRequirementForProperty( which );
   else return AngleImp::stype();
 }
 
-const char* AngleImp::iconForProperty( uint which ) const
+const char* AngleImp::iconForProperty( int which ) const
 {
   int numprop = 0;
   if ( which < Parent::numberOfProperties() )
@@ -128,7 +131,7 @@ const char* AngleImp::iconForProperty( uint which ) const
   return "";
 }
 
-ObjectImp* AngleImp::property( uint which, const KigDocument& w ) const
+ObjectImp* AngleImp::property( int which, const KigDocument& w ) const
 {
   int numprop = 0;
   if ( which < Parent::numberOfProperties() )
@@ -147,7 +150,7 @@ ObjectImp* AngleImp::property( uint which, const KigDocument& w ) const
   return new InvalidImp;
 }
 
-const double AngleImp::size() const
+double AngleImp::size() const
 {
   return mangle;
 }
@@ -189,14 +192,14 @@ bool VectorImp::inRect( const Rect& r, int width, const KigWidget& w ) const
   return lineInRect( r, mdata.a, mdata.b, width, this, w );
 }
 
-const uint VectorImp::numberOfProperties() const
+int VectorImp::numberOfProperties() const
 {
   return Parent::numberOfProperties() + 5;
 }
 
-const QCStringList VectorImp::propertiesInternalNames() const
+const QByteArrayList VectorImp::propertiesInternalNames() const
 {
-  QCStringList ret = Parent::propertiesInternalNames();
+  QByteArrayList ret = Parent::propertiesInternalNames();
   ret << "length";
   ret << "vect-mid-point";
   ret << "length-x";
@@ -206,9 +209,9 @@ const QCStringList VectorImp::propertiesInternalNames() const
   return ret;
 }
 
-const QCStringList VectorImp::properties() const
+const QByteArrayList VectorImp::properties() const
 {
-  QCStringList ret = Parent::properties();
+  QByteArrayList ret = Parent::properties();
   ret << I18N_NOOP( "Length" );
   ret << I18N_NOOP( "Midpoint" );
   ret << I18N_NOOP( "X length" );
@@ -218,14 +221,14 @@ const QCStringList VectorImp::properties() const
   return ret;
 }
 
-const ObjectImpType* VectorImp::impRequirementForProperty( uint which ) const
+const ObjectImpType* VectorImp::impRequirementForProperty( int which ) const
 {
   if ( which < Parent::numberOfProperties() )
     return Parent::impRequirementForProperty( which );
   else return VectorImp::stype();
 }
 
-const char* VectorImp::iconForProperty( uint which ) const
+const char* VectorImp::iconForProperty( int which ) const
 {
   if ( which < Parent::numberOfProperties() )
     return Parent::iconForProperty( which );
@@ -243,7 +246,7 @@ const char* VectorImp::iconForProperty( uint which ) const
   return "";
 }
 
-ObjectImp* VectorImp::property( uint which, const KigDocument& w ) const
+ObjectImp* VectorImp::property( int which, const KigDocument& w ) const
 {
   if ( which < Parent::numberOfProperties() )
     return Parent::property( which, w );
@@ -281,7 +284,7 @@ void VectorImp::visit( ObjectImpVisitor* vtor ) const
   vtor->visit( this );
 }
 
-const double VectorImp::length() const
+double VectorImp::length() const
 {
   return ( mdata.a - mdata.b ).length();
 }
@@ -311,10 +314,17 @@ ArcImp* ArcImp::copy() const
 ObjectImp* ArcImp::transform( const Transformation& t ) const
 {
   //
-  // we don't have conic arcs! So it is invalid to transform an arc
-  // with a nonhomothetic transformation
+  // if the transformation is nonhomothetic, the result is a conic arc,
+  // not a circular arc
   //
-  if ( ! t.isHomothetic() ) return new InvalidImp();
+  if ( ! t.isHomothetic() )
+  {
+    //CircleImp support = CircleImp( mcenter, mradius );
+    ConicCartesianData data = CircleImp( mcenter, mradius ).cartesianData();
+    //return new InvalidImp();
+    ConicArcImp conicarc = ConicArcImp( data, msa, ma );
+    return conicarc.transform ( t );
+  }
 
   Coordinate nc = t.apply( mcenter );
   double nr = t.apply( mradius );
@@ -356,14 +366,14 @@ bool ArcImp::valid() const
   return true;
 }
 
-const uint ArcImp::numberOfProperties() const
+int ArcImp::numberOfProperties() const
 {
-  return Parent::numberOfProperties() + 9;
+  return Parent::numberOfProperties() + 10;
 }
 
-const QCStringList ArcImp::properties() const
+const QByteArrayList ArcImp::properties() const
 {
-  QCStringList ret = Parent::properties();
+  QByteArrayList ret = Parent::properties();
   ret << I18N_NOOP( "Center" );
   ret << I18N_NOOP( "Radius" );
   ret << I18N_NOOP( "Angle" );
@@ -371,15 +381,16 @@ const QCStringList ArcImp::properties() const
   ret << I18N_NOOP( "Angle in Radians" );
   ret << I18N_NOOP( "Sector Surface" );
   ret << I18N_NOOP( "Arc Length" );
+  ret << I18N_NOOP( "Support Circle" );
   ret << I18N_NOOP( "First End Point" );
   ret << I18N_NOOP( "Second End Point" );
   assert( ret.size() == ArcImp::numberOfProperties() );
   return ret;
 }
 
-const QCStringList ArcImp::propertiesInternalNames() const
+const QByteArrayList ArcImp::propertiesInternalNames() const
 {
-  QCStringList ret = Parent::propertiesInternalNames();
+  QByteArrayList ret = Parent::propertiesInternalNames();
   ret << "center";
   ret << "radius";
   ret << "angle";
@@ -387,12 +398,13 @@ const QCStringList ArcImp::propertiesInternalNames() const
   ret << "angle-radians";
   ret << "sector-surface";
   ret << "arc-length";
+  ret << "support";
   ret << "end-point-A";
   ret << "end-point-B";
   return ret;
 }
 
-const char* ArcImp::iconForProperty( uint which ) const
+const char* ArcImp::iconForProperty( int which ) const
 {
   int numprop = 0;
   if ( which < Parent::numberOfProperties() )
@@ -415,11 +427,13 @@ const char* ArcImp::iconForProperty( uint which ) const
     return "";
   else if ( which == Parent::numberOfProperties() + numprop++ )
     return "";
+  else if ( which == Parent::numberOfProperties() + numprop++ )
+    return "";
   else assert( false );
   return "";
 }
 
-ObjectImp* ArcImp::property( uint which, const KigDocument& d ) const
+ObjectImp* ArcImp::property( int which, const KigDocument& d ) const
 {
   int numprop = 0;
   if ( which < Parent::numberOfProperties() )
@@ -438,6 +452,8 @@ ObjectImp* ArcImp::property( uint which, const KigDocument& d ) const
     return new DoubleImp( sectorSurface() );
   else if ( which == Parent::numberOfProperties() + numprop++ )
     return new DoubleImp( mradius * ma );
+  else if ( which == Parent::numberOfProperties() + numprop++ )
+    return new CircleImp( mcenter, mradius );
   else if ( which == Parent::numberOfProperties() + numprop++ )
     return new PointImp( firstEndPoint() );
   else if ( which == Parent::numberOfProperties() + numprop++ )
@@ -446,12 +462,12 @@ ObjectImp* ArcImp::property( uint which, const KigDocument& d ) const
   return new InvalidImp;
 }
 
-const double ArcImp::sectorSurface() const
+double ArcImp::sectorSurface() const
 {
   return mradius * mradius * ma / 2;
 }
 
-const ObjectImpType* ArcImp::impRequirementForProperty( uint which ) const
+const ObjectImpType* ArcImp::impRequirementForProperty( int which ) const
 {
   if ( which < Parent::numberOfProperties() )
     return Parent::impRequirementForProperty( which );
@@ -624,26 +640,46 @@ bool ArcImp::internalContainsPoint( const Coordinate& p, double threshold ) cons
   return isOnArc( p, mcenter, mradius, msa, ma, threshold );
 }
 
-bool AngleImp::isPropertyDefinedOnOrThroughThisImp( uint which ) const
+bool AngleImp::isPropertyDefinedOnOrThroughThisImp( int which ) const
 {
   if ( which < Parent::numberOfProperties() )
     return Parent::isPropertyDefinedOnOrThroughThisImp( which );
   return false;
 }
 
-bool VectorImp::isPropertyDefinedOnOrThroughThisImp( uint which ) const
+bool VectorImp::isPropertyDefinedOnOrThroughThisImp( int which ) const
 {
   return Parent::isPropertyDefinedOnOrThroughThisImp( which );
 }
 
-bool ArcImp::isPropertyDefinedOnOrThroughThisImp( uint which ) const
+bool ArcImp::isPropertyDefinedOnOrThroughThisImp( int which ) const
 {
+  int pnum = 0;
+
   if ( which < Parent::numberOfProperties() )
     return Parent::isPropertyDefinedOnOrThroughThisImp( which );
-  else if ( which == Parent::numberOfProperties() )
-    return true;
-  else
-    return false;
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return false;  // center
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return false;   // radius
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return false;   // angle
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return false;   // angle in degrees
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return false;   // angle in radians
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return false;   // sector surface
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return false;   // arc length
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return false;    // support
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return true;    // first end-point
+  else if ( which == Parent::numberOfProperties() + pnum++ )
+    return true;    // second end-point
+  else return false;
+  return false;
 }
 
 Rect AngleImp::surroundingRect() const
