@@ -1,7 +1,7 @@
 /***************************************************************************
                           wqscore.cpp  -  description
                              -------------------
-   copyright            : (C) 2003 by Peter Hedlund
+   copyright            : (C) 2003, 2007 by Peter Hedlund
    email                : peter.hedlund@kdemail.net
  ***************************************************************************/
 
@@ -25,11 +25,6 @@ WQScore::WQScore()
   m_answerCount = 0;
 }
 
-
-WQScore::~WQScore()
-{}
-
-
 void WQScore::setAsPercent(bool p)
 {
   m_percent = p;
@@ -38,7 +33,6 @@ void WQScore::setAsPercent(bool p)
 void WQScore::setQuestionCount(int c)
 {
   m_questionCount = c;
-  //m_percent = false;
   m_error = 0;
   m_correct = 0;
   m_answerCount = 0;
@@ -46,7 +40,6 @@ void WQScore::setQuestionCount(int c)
 
 void WQScore::countIncrement(CountDirection d)
 {
-
   if (d != cdNone)
     m_answerCount++;
   if (d == cdCorrect)
@@ -67,9 +60,9 @@ QString WQScore::answerText()
     if (m_answerCount > 0)
     {
       if (f < 1)
-        s = s.setNum(f, 'f', 1) + "%"; //for long lists (i.e. each question is less than 1%) we show one decimal 
+        s = s.setNum(f, 'f', 1) + '%'; //for long lists (i.e. each question is less than 1%) we show one decimal
       else
-        s = s.setNum(f, 'f', 0) + "%";
+        s = s.setNum(f, 'f', 0) + '%';
     }
     else
       s= valueToString(m_answerCount);
@@ -90,7 +83,7 @@ QString WQScore::correctText()
     if (m_correct > 0)
     {
       float f = ((m_correct * 100.0) / m_answerCount);
-      s = s.setNum(f, 'f', 0) + "%";
+      s = s.setNum(f, 'f', 0) + '%';
     }
     else
       s = valueToString(m_correct);
@@ -112,7 +105,7 @@ QString WQScore::errorText()
     if (m_error > 0)
     {
       float f = ((m_error * 100.0) / m_answerCount);
-      s = s.setNum(f, 'f', 0) + "%";
+      s = s.setNum(f, 'f', 0) + '%';
     }
     else
       s = valueToString(m_error);
