@@ -119,7 +119,7 @@ MarbleModel::MarbleModel( QObject *parent )
       d( new MarbleModelPrivate( this ) )
 {
     d->m_dataFacade = new MarbleDataFacade( this );
-    d->m_layerManager = new LayerManager( d->m_dataFacade );
+    d->m_layerManager = new LayerManager( d->m_dataFacade, this );
 
     // FIXME: more on the spot update names and API
     connect ( d->m_layerManager,      SIGNAL( floatItemsChanged() ),
@@ -141,8 +141,6 @@ MarbleModel::MarbleModel( QObject *parent )
     d->m_texmapper = 0;
     d->m_veccomposer = new VectorComposer();
 
-    d->m_placeMarkLayout   = 0;
-
     d->m_gridmap      = new GridMap();
     d->m_texcolorizer = new TextureColorizer( MarbleDirs::path( "seacolors.leg" ),
                                               MarbleDirs::path( "landcolors.leg" ) );
@@ -155,6 +153,13 @@ MarbleModel::MarbleModel( QObject *parent )
     d->m_placemarkmodel = new MarblePlacemarkModel( d->m_placemarkmanager, this );
     d->m_placemarkselectionmodel = new QItemSelectionModel( d->m_placemarkmodel );
 
+    d->m_placeMarkLayout = new PlaceMarkLayout( this );
+    connect( d->m_placemarkselectionmodel, SIGNAL( selectionChanged( QItemSelection,
+                                                                        QItemSelection) ),
+                d->m_placeMarkLayout, SLOT( requestStyleReset() ) );
+    connect( d->m_placemarkmodel, SIGNAL( modelReset() ),
+                d->m_placeMarkLayout, SLOT( requestStyleReset() ) );
+
     d->m_placemarkmanager->loadStandardPlaceMarks();
 
     d->m_gpxFileModel = new GpxFileModel( this );
@@ -187,7 +192,7 @@ MarbleModel::MarbleModel( QObject *parent )
 
 MarbleModel::~MarbleModel()
 {
-    qDebug() << "MarbleModel::~MarbleModel";
+//    qDebug() << "MarbleModel::~MarbleModel";
     
     delete d->m_texmapper;
 
@@ -204,7 +209,11 @@ MarbleModel::~MarbleModel()
     delete d->m_timer;
     delete d->m_layerManager;
     delete d->m_dataFacade;
+    delete d->m_layerDecorator;
+    delete d->m_sunLocator;
+    delete d->m_dateTime;
     delete d;
+    qDebug() << "Model deleted:" << this;
 }
 
 bool MarbleModel::showGps() const
@@ -351,15 +360,6 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
         }
     }
 
-    if ( d->m_placeMarkLayout == 0) {
-        d->m_placeMarkLayout = new PlaceMarkLayout( this );
-        connect( d->m_placemarkselectionmodel, SIGNAL( selectionChanged( QItemSelection,
-                                                                         QItemSelection) ),
-                 d->m_placeMarkLayout, SLOT( requestStyleReset() ) );
-        connect( d->m_placemarkmodel, SIGNAL( modelReset() ),
-                 d->m_placeMarkLayout, SLOT( requestStyleReset() ) );
-    }
-
     d->m_placeMarkLayout->requestStyleReset();
     // FIXME: To be removed after MapTheme / KML refactoring
 
@@ -627,7 +627,6 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
                                viewParams->canvasImage()->size(),
                                viewParams,
                                viewParams->viewport()->boundingBox() );
-
     d->m_layerManager->renderLayers( painter, viewParams );
 }
 
@@ -716,6 +715,8 @@ void MarbleModel::update()
 
 ExtDateTime* MarbleModel::dateTime() const
 {
+//    qDebug() << "In dateTime, model:" << this;
+//    qDebug() << d << ":" << d->m_dateTime;
     return d->m_dateTime;
 }
 
